@@ -2,12 +2,16 @@
  *    Example program for the Allegro library, by Shawn Hargreaves.
  *
  *    This program demonstrates how to use the timer routines.
- *    These can be a bit of a pain, because you have to be sure you lock 
- *    all the memory that is used inside your interrupt handlers.
+ *    These can be a bit of a pain, because you have to be sure
+ *    you lock all the memory that is used inside your interrupt
+ *    handlers.  The first part of the example shows a basic use of
+ *    timing using the blocking function rest(). The second part
+ *    shows how to use three timers with different frequencies in
+ *    a non blocking way.
  */
 
 
-#include "allegro.h"
+#include <allegro.h>
 
 
 
@@ -24,7 +28,7 @@ void inc_x(void)
    x++;
 }
 
-END_OF_FUNCTION(inc_x);
+END_OF_FUNCTION(inc_x)
 
 
 
@@ -34,7 +38,7 @@ void inc_y(void)
    y++;
 }
 
-END_OF_FUNCTION(inc_y);
+END_OF_FUNCTION(inc_y)
 
 
 
@@ -44,7 +48,7 @@ void inc_z(void)
    z++;
 }
 
-END_OF_FUNCTION(inc_z);
+END_OF_FUNCTION(inc_z)
 
 
 
@@ -68,17 +72,23 @@ int main(void)
    set_palette(desktop_palette);
    clear_to_color(screen, makecol(255, 255, 255));
 
-   textprintf_centre_ex(screen, font, SCREEN_W/2, 8, makecol(0, 0, 0), makecol(255, 255, 255), "Driver: %s", timer_driver->name);
+   textprintf_centre_ex(screen, font, SCREEN_W/2, 8, makecol(0, 0, 0),
+			makecol(255, 255, 255), "Driver: %s",
+			timer_driver->name);
 
    /* use rest() to delay for a specified number of milliseconds */
-   textprintf_centre_ex(screen, font, SCREEN_W/2, 48, makecol(0, 0, 0), makecol(255, 255, 255), "Timing five seconds:");
+   textprintf_centre_ex(screen, font, SCREEN_W/2, 48, makecol(0, 0, 0),
+			makecol(255, 255, 255), "Timing five seconds:");
 
    for (c=1; c<=5; c++) {
-      textprintf_centre_ex(screen, font, SCREEN_W/2, 62+c*10, makecol(0, 0, 0), makecol(255, 255, 255), "%d", c);
+      textprintf_centre_ex(screen, font, SCREEN_W/2, 62+c*10,
+			   makecol(0, 0, 0), makecol(255, 255, 255), "%d", c);
       rest(1000);
    }
 
-   textprintf_centre_ex(screen, font, SCREEN_W/2, 142, makecol(0, 0, 0), makecol(255, 255, 255), "Press a key to set up interrupts");
+   textprintf_centre_ex(screen, font, SCREEN_W/2, 142, makecol(0, 0, 0),
+			makecol(255, 255, 255),
+			"Press a key to set up interrupts");
    readkey();
 
    /* all variables and code used inside interrupt handlers must be locked */
@@ -100,9 +110,10 @@ int main(void)
 
    /* the interrupts are now active... */
    while (!keypressed())
-      textprintf_centre_ex(screen, font, SCREEN_W/2, 176, makecol(0, 0, 0), makecol(255, 255, 255), "x=%d, y=%d, z=%d", x, y, z);
+      textprintf_centre_ex(screen, font, SCREEN_W/2, 176, makecol(0, 0, 0),
+			   makecol(255, 255, 255), "x=%d, y=%d, z=%d", x, y, z);
 
    return 0;
 }
 
-END_OF_MAIN();
+END_OF_MAIN()
