@@ -27,12 +27,17 @@
    #include <stddef.h>
    #include <stdlib.h>
    #include <time.h>
+   #include <string.h>
 #endif
 
 #if (defined DEBUGMODE) && (defined FORTIFY)
    #include <fortify/fortify.h>
 #endif
 
+#if (defined DEBUGMODE) && (defined DMALLOC)
+   #include <dmalloc.h>
+#endif
+
 #include "internal/alconfig.h"
 
 #ifdef __cplusplus
@@ -40,11 +45,11 @@
 #endif
 
 #define ALLEGRO_VERSION          4
-#define ALLEGRO_SUB_VERSION      1
-#define ALLEGRO_WIP_VERSION      12
-#define ALLEGRO_VERSION_STR      "4.1.12 (WIP)"
-#define ALLEGRO_DATE_STR         "2003"
-#define ALLEGRO_DATE             20031110    /* yyyymmdd */
+#define ALLEGRO_SUB_VERSION      2
+#define ALLEGRO_WIP_VERSION      2
+#define ALLEGRO_VERSION_STR      "4.2.2"
+#define ALLEGRO_DATE_STR         "2007"
+#define ALLEGRO_DATE             20070722    /* yyyymmdd */
 
 /*******************************************/
 /************ Some global stuff ************/
