@@ -12,7 +12,7 @@
 #
 #  Note: if you write datestamp in the archive_name field, then the
 #  resulting archive will be datestamped. This is in particular useful
-#  for making CVS snapshots.
+#  for making SVN snapshots.
 
 
 if [ $# -lt 1 -o $# -gt 2 ]; then
@@ -20,6 +20,12 @@ if [ $# -lt 1 -o $# -gt 2 ]; then
    exit 1
 fi
 
+if [ -n "$2" ] && [ ! -r ../"$2" ]; then
+   echo "Previous archive $2 not in parent directory, aborting"
+   exit 1
+fi
+
+
 
 # strip off the path and extension from our arguments
 name=$(echo "$1" | sed -e 's/.*[\\\/]//; s/\.zip//')
@@ -39,7 +45,7 @@ for file in makefile.*; do
    rm _tmpfile
 done
 
-# fix some wrong permissions in the CVS repository
+# fix some wrong permissions in the SVN repository
 chmod +x misc/asmdef.sh misc/fixdll.sh
 
 
@@ -96,10 +102,10 @@ echo "Generating MSVC dependencies..."
 make depend UNIX_TOOLS=1 CC=gcc
 
 
-# generate dependencies for MinGW32
-echo "Generating MinGW32 dependencies..."
+# generate dependencies for MinGW
+echo "Generating MinGW dependencies..."
 
-./fix.sh mingw32 --quick
+./fix.sh mingw --quick
 
 make depend UNIX_TOOLS=1 CC=gcc
 
@@ -139,10 +145,6 @@ make depend UNIX_TOOLS=1 CC=gcc
 # generate the DLL export definition files for Windows compilers
 misc/fixdll.sh
 
-# running autoheader
-echo "Running autoheader to generate configure header..."
-autoheader
-
 # running autoconf
 echo "Running autoconf to generate configure script..."
 autoconf || exit 1
@@ -171,7 +173,7 @@ rm _makedoc.exe
 
 
 # create language.dat and keyboard.dat files
-misc/mkdata.sh
+misc/mkdata.sh || exit 1
 
 
 # convert files to djgpp format for distribution
@@ -202,10 +204,17 @@ scan_for_empties "."
 
 # build the main zip archive
 echo "Creating $name.zip..."
-cd ..
-if [ -f $name.zip ]; then rm $name.zip; fi
-rm -rf allegro/autom4te*
-find allegro -iname "CVS" -prune -o -iname ".*" -prune -o -iname "*.rej" -prune -o -iname "*.orig" -prune -o -print | zip -9 $name.zip -@
+if [ -f .dist/$name.zip ]; then rm .dist/$name.zip; fi
+rm -rf ./autom4te*
+ZIP_FILES=`find . -type f "(" -path "*/.*" -prune -o -iname "*.rej" \
+    -prune -o -iname "*.orig" -prune -o -print ")"`
+mkdir -p .dist/allegro
+cp -a --parents $ZIP_FILES .dist/allegro
+
+# from now on, the scripts runs inside .dist
+cd .dist
+
+zip -9  -r $name.zip allegro
 
 
 # generate the manifest file
@@ -225,7 +234,7 @@ if [ $# -eq 2 ]; then
 
    echo "Inflating previous version ($2)..."
    mkdir previous
-   unzip -q "$2" -d previous
+   unzip -q ../../"$2" -d previous || exit 1
 
    echo "Generating diffs..."
    diff -U 3 -N --recursive previous/ current/ > $name.diff
