@@ -207,7 +207,7 @@ static FILE *assert_file = NULL;
 static FILE *trace_file = NULL;
 
 static int (*assert_handler)(AL_CONST char *msg) = NULL;
-static int (*trace_handler)(AL_CONST char *msg) = NULL;
+int (*_al_trace_handler)(AL_CONST char *msg) = NULL;
 
 
 /* Module linking system stuff: if an object file is linked in, then its
@@ -232,11 +232,17 @@ static struct al_exit_func *exit_func_list = NULL;
 
 
 /* _get_allegro_version:
- *  Retrieves the library version.
+ *  Retrieves the library version.  This is an obsolete definition which should
+ *  only be used by Allegro 4.2.0 binaries, which have a call to this function
+ *  embedded.
  */
 int _get_allegro_version(void)
 {
-   return MAKE_VERSION(ALLEGRO_VERSION, ALLEGRO_SUB_VERSION, ALLEGRO_WIP_VERSION);
+#if ALLEGRO_VERSION == 4 && ALLEGRO_SUB_VERSION == 2
+   return MAKE_VERSION(ALLEGRO_VERSION, ALLEGRO_SUB_VERSION, 0);
+#else
+#error _get_allegro_version should not exist in other versions of Allegro
+#endif
 }
 
 
@@ -254,7 +260,7 @@ void _add_exit_func(void (*func)(void), AL_CONST char *desc)
       if (n->funcptr == func)
 	 return;
 
-   n = malloc(sizeof(struct al_exit_func));
+   n = _AL_MALLOC(sizeof(struct al_exit_func));
    if (!n)
       return;
 
@@ -279,7 +285,7 @@ void _remove_exit_func(void (*func)(void))
 	    prev->next = iter->next;
 	 else
 	    exit_func_list = iter->next;
-	 free(iter);
+	 _AL_FREE(iter);
 	 return;
       }
       prev = iter;
@@ -303,6 +309,9 @@ static void allegro_exit_stub(void)
 
 /* _install_allegro:
  *  Initialises the Allegro library, activating the system driver.
+ *
+ *  This is non-static because 4.2.0 binaries reference it.  Newer binaries
+ *  should only call this through _install_allegro_version_check().
  */
 int _install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*func)(void)))
 {
@@ -411,13 +420,59 @@ int _install_allegro(int system_id, int *errno_ptr, int (*atexit_ptr)(void (*fun
 
 
 
+/* _install_allegro_version_check:
+ *  Initialises the Allegro library, but return with an error if an
+ *  incompatible version is found.
+ */
+int _install_allegro_version_check(int system_id, int *errno_ptr,
+   int (*atexit_ptr)(void (*func)(void)), int version)
+{
+   int r = _install_allegro(system_id, errno_ptr, atexit_ptr);
+
+   int build_wip = version & 255;
+   int build_ver = version & ~255;
+
+   int version_ok;
+
+   if (r != 0) {
+      /* failed */
+      return r;
+   }
+
+#if ALLEGRO_SUB_VERSION & 1
+   /* This is a WIP runtime, so enforce strict compatibility. */
+   version_ok = version == MAKE_VERSION(ALLEGRO_VERSION, ALLEGRO_SUB_VERSION, ALLEGRO_WIP_VERSION);
+#else
+   /* This is a stable runtime, so the runtime should be at least as new
+    * as the build headers (otherwise we may get a crash, since some
+    * functions may have been used which aren't available in this runtime).
+    */
+   version_ok = (MAKE_VERSION(ALLEGRO_VERSION, ALLEGRO_SUB_VERSION, 0) == build_ver) &&
+      (ALLEGRO_WIP_VERSION >= build_wip);
+#endif
+
+   if (!version_ok) {
+      uszprintf(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text(
+         "The detected dynamic Allegro library (%d.%d.%d) is "
+         "not compatible with this program (%d.%d.%d)."),
+         ALLEGRO_VERSION, ALLEGRO_SUB_VERSION, ALLEGRO_WIP_VERSION,
+         build_ver >> 16, (build_ver >> 8) & 255, build_wip);
+      return -1;
+   }
+   return 0;
+}
+
+
+
 /* allegro_exit:
  *  Closes down the Allegro system.
  */
 void allegro_exit(void)
 {
    while (exit_func_list) {
-      (*(exit_func_list->funcptr))();
+      void (*func)(void) = exit_func_list->funcptr;
+      _remove_exit_func(func);
+      (*func)();
    }
 
    if (system_driver) {
@@ -426,7 +481,7 @@ void allegro_exit(void)
    }
 
    if (_scratch_mem) {
-      free(_scratch_mem);
+      _AL_FREE(_scratch_mem);
       _scratch_mem = NULL;
       _scratch_mem_size = 0;
    }
@@ -439,8 +494,8 @@ void allegro_exit(void)
  */
 void allegro_message(AL_CONST char *msg, ...)
 {
-   char *buf = malloc(ALLEGRO_MESSAGE_SIZE);
-   char *tmp = malloc(ALLEGRO_MESSAGE_SIZE);
+   char *buf = _AL_MALLOC_ATOMIC(ALLEGRO_MESSAGE_SIZE);
+   char *tmp = _AL_MALLOC_ATOMIC(ALLEGRO_MESSAGE_SIZE);
    va_list ap;
    ASSERT(msg);
    va_start(ap, msg);
@@ -452,8 +507,8 @@ void allegro_message(AL_CONST char *msg, ...)
    else
       fputs(uconvert(buf, U_CURRENT, tmp, U_ASCII_CP, ALLEGRO_MESSAGE_SIZE), stdout);
 
-   free(buf);
-   free(tmp);
+   _AL_FREE(buf);
+   _AL_FREE(tmp);
 }
 
 
@@ -590,8 +645,8 @@ void al_trace(AL_CONST char *msg, ...)
    vsprintf(buf, msg, ap);
    va_end(ap);
 
-   if (trace_handler) {
-      if (trace_handler(buf))
+   if (_al_trace_handler) {
+      if (_al_trace_handler(buf))
 	 return;
    }
 
@@ -634,7 +689,7 @@ void register_assert_handler(int (*handler)(AL_CONST char *msg))
  */
 void register_trace_handler(int (*handler)(AL_CONST char *msg))
 {
-   trace_handler = handler;
+   _al_trace_handler = handler;
 }
 
 
@@ -643,7 +698,7 @@ void register_trace_handler(int (*handler)(AL_CONST char *msg))
  *  Wrapper for when a program needs to manipulate memory that has been
  *  allocated by the Allegro DLL.
  */
-void *_al_malloc(int size)
+void *_al_malloc(size_t size)
 {
    return malloc(size);
 }
@@ -665,7 +720,7 @@ void _al_free(void *mem)
  *  Wrapper for when a program needs to manipulate memory that has been
  *  allocated by the Allegro DLL.
  */
-void *_al_realloc(void *mem, int size)
+void *_al_realloc(void *mem, size_t size)
 {
    return realloc(mem, size);
 }
