@@ -39,6 +39,9 @@
    #include <pwd.h>                 /* for tilde expansion */
 #endif
 
+#ifdef ALLEGRO_WINDOWS
+   #include "winalleg.h" /* for GetTempPath */
+#endif
 
 
 /* permissions to use when opening files */
@@ -71,6 +74,7 @@ static PACKFILE_VTABLE normal_vtable;
 
 static PACKFILE *pack_fopen_special_file(AL_CONST char *filename, AL_CONST char *mode);
 
+static int file_encoding = U_ASCII;
 
 
 #define FA_DAT_FLAGS  (FA_RDONLY | FA_ARCH)
@@ -196,7 +200,7 @@ char *canonicalize_filename(char *dest, AL_CONST char *filename, int size)
 	     * pointers. This code is safe on the assumption that ASCII is
 	     * the most efficient encoding, but wasteful of memory */
 	    userlen = tail - username + ucwidth('\0');
-	    ascii_username = malloc(userlen);
+	    ascii_username = _AL_MALLOC_ATOMIC(userlen);
 
 	    if (ascii_username) {
 	       /* convert the username to ASCII, find the password entry,
@@ -212,7 +216,7 @@ char *canonicalize_filename(char *dest, AL_CONST char *filename, int size)
 		      (strcmp(pwd->pw_name, ascii_username) != 0))
 		  ;
 
-	       free(ascii_username);
+	       _AL_FREE(ascii_username);
 
 	       if (pwd)
 		  home = strdup(pwd->pw_dir);
@@ -227,7 +231,7 @@ char *canonicalize_filename(char *dest, AL_CONST char *filename, int size)
 	  */
 	 if (home) {
 	    do_uconvert(home, U_ASCII, buf+pos, U_CURRENT, sizeof(buf)-pos);
-	    free(home);
+	    _AL_FREE(home);
 	    pos = ustrsize(buf);
 	    filename = tail;
 	    goto no_relativisation;
@@ -375,7 +379,7 @@ char *make_relative_filename(char *dest, AL_CONST char *path, AL_CONST char *fil
 
    my_filename = ustrdup(filename);
    if (!my_filename) {
-      free(my_path);
+      _AL_FREE(my_path);
       return NULL;
    }
 
@@ -420,8 +424,8 @@ char *make_relative_filename(char *dest, AL_CONST char *path, AL_CONST char *fil
       /* Bail out if previously something went wrong (eg. user supplied
        * paths are not canonical and we can't understand them). */
       if (!reduced_path) {
-	 free(my_path);
-	 free(my_filename);
+	 _AL_FREE(my_path);
+	 _AL_FREE(my_filename);
 	 return NULL;
       }
       /* Otherwise, we are in the latter case and need to count the number
@@ -443,16 +447,16 @@ char *make_relative_filename(char *dest, AL_CONST char *path, AL_CONST char *fil
    /* Bail out if previously something went wrong (eg. user supplied
     * paths are not canonical and we can't understand them). */
    if (!reduced_filename) {
-      free(my_path);
-      free(my_filename);
+      _AL_FREE(my_path);
+      _AL_FREE(my_filename);
       return NULL;
    }
 
    ustrzcat(dest, size, reduced_filename);
    ustrzcat(dest, size, get_filename(filename));
 
-   free(my_path);
-   free(my_filename);
+   _AL_FREE(my_path);
+   _AL_FREE(my_filename);
 
    /* Harmonize path separators. */
    return fix_filename_slashes(dest);
@@ -668,6 +672,26 @@ void put_backslash(char *filename)
  ***************************************************/
 
 
+/* set_file_encoding:
+ *  Sets the encoding to use for filenames. By default, UTF8 is assumed.
+ */
+void set_file_encoding(int encoding)
+{
+    file_encoding = encoding;
+}
+
+
+
+/* get_file_encoding:
+ *  Returns the encoding currently assumed for filenames.
+ */
+int get_file_encoding(void)
+{
+    return file_encoding ;
+}
+
+
+
 /* file_exists:
  *  Checks whether a file matching the given name and attributes exists,
  *  returning non zero if it does. The file attribute may contain any of
@@ -722,12 +746,12 @@ int exists(AL_CONST char *filename)
 
 
 
-/* file_size:
+/* file_size_ex:
  *  Returns the size of a file, in bytes.
  *  If the file does not exist or an error occurs, it will return zero
  *  and store the system error code in errno.
  */
-long file_size(AL_CONST char *filename)
+uint64_t file_size_ex(AL_CONST char *filename)
 {
    ASSERT(filename);
    if (ustrchr(filename, '#')) {
@@ -744,7 +768,26 @@ long file_size(AL_CONST char *filename)
    if (!_al_file_isok(filename))
       return 0;
 
-   return _al_file_size(filename);
+   return _al_file_size_ex(filename);
+}
+
+
+
+/* For binary compatibility with 4.2.0. */
+long file_size(AL_CONST char *filename)
+{
+   return file_size_ex(filename);
+}
+
+
+
+/* For binary compatibility with 4.2.0.
+ * This is an internal symbol and only required because _al_file_size was
+ * exposed in the Windows DLL.
+ */
+long _al_file_size(AL_CONST char *filename)
+{
+   return _al_file_size_ex(filename);
 }
 
 
@@ -777,7 +820,7 @@ int delete_file(AL_CONST char *filename)
    if (!_al_file_isok(filename))
       return -1;
 
-   if (unlink(uconvert_toascii(filename, tmp)) != 0) {
+   if (_al_unlink(uconvert_tofilename(filename, tmp)) != 0) {
       *allegro_errno = errno;
       return -1;
    }
@@ -1030,7 +1073,7 @@ int set_allegro_resource_path(int priority, AL_CONST char *path)
       if (node && priority == node->priority)
 	 new_node = node;
       else {
-	 new_node = malloc(sizeof(RESOURCE_PATH));
+	 new_node = _AL_MALLOC(sizeof(RESOURCE_PATH));
 	 if (!new_node)
 	    return 0;
          
@@ -1063,7 +1106,7 @@ int set_allegro_resource_path(int priority, AL_CONST char *path)
 	  else
 	     resource_path_list = node->next;
           
-	  free(node);
+	  _AL_FREE(node);
           
 	  if (!resource_path_list)
 	     _remove_exit_func(destroy_resource_path_list);
@@ -1086,7 +1129,7 @@ static void destroy_resource_path_list(void)
    
    while (node) {
       resource_path_list = node->next;
-      free(node);
+      _AL_FREE(node);
       node = resource_path_list;
    }
 }
@@ -1186,7 +1229,7 @@ int find_allegro_resource(char *dest, AL_CONST char *resource, AL_CONST char *ex
 
    /* try any extra environment variable that the parameters say to use */
    if (envvar) {
-      s = getenv(uconvert_toascii(envvar, tmp));
+      s = getenv(uconvert_tofilename(envvar, tmp));
 
       if (s) {
 	 do_uconvert(s, U_ASCII, path, U_CURRENT, sizeof(path)-ucwidth(OTHER_PATH_SEPARATOR));
@@ -1482,7 +1525,7 @@ static int clone_password(PACKFILE *f)
    ASSERT(f->is_normal_packfile);
 
    if (the_password[0]) {
-      if ((f->normal.passdata = malloc(strlen(the_password)+1)) == NULL) {
+      if ((f->normal.passdata = _AL_MALLOC_ATOMIC(strlen(the_password)+1)) == NULL) {
 	 *allegro_errno = ENOMEM;
 	 return FALSE;
       }
@@ -1507,9 +1550,9 @@ static PACKFILE *create_packfile(int is_normal_packfile)
    PACKFILE *f;
 
    if (is_normal_packfile)
-      f = malloc(sizeof(PACKFILE));
+      f = _AL_MALLOC(sizeof(PACKFILE));
    else
-      f = malloc(sizeof(PACKFILE) - sizeof(struct _al_normal_packfile_details));
+      f = _AL_MALLOC(sizeof(PACKFILE) - sizeof(struct _al_normal_packfile_details));
 
    if (f == NULL) {
       *allegro_errno = ENOMEM;
@@ -1560,7 +1603,7 @@ static void free_packfile(PACKFILE *f)
 	 ASSERT(!f->normal.passpos);
       }
 
-      free(f);
+      _AL_FREE(f);
    }
 }
 
@@ -1781,14 +1824,14 @@ PACKFILE *pack_fopen(AL_CONST char *filename, AL_CONST char *mode)
 
 #ifndef ALLEGRO_MPW
    if (strpbrk(mode, "wW"))  /* write mode? */
-      fd = open(uconvert_toascii(filename, tmp), O_WRONLY | O_BINARY | O_CREAT | O_TRUNC, OPEN_PERMS);
+      fd = _al_open(uconvert_tofilename(filename, tmp), O_WRONLY | O_BINARY | O_CREAT | O_TRUNC, OPEN_PERMS);
    else
-      fd = open(uconvert_toascii(filename, tmp), O_RDONLY | O_BINARY, OPEN_PERMS);
+      fd = _al_open(uconvert_tofilename(filename, tmp), O_RDONLY | O_BINARY, OPEN_PERMS);
 #else
    if (strpbrk(mode, "wW"))  /* write mode? */
-      fd = _al_open(uconvert_toascii(filename, tmp), O_WRONLY | O_BINARY | O_CREAT | O_TRUNC);
+      fd = _al_open(uconvert_tofilename(filename, tmp), O_WRONLY | O_BINARY | O_CREAT | O_TRUNC);
    else
-      fd = _al_open(uconvert_toascii(filename, tmp), O_RDONLY | O_BINARY);
+      fd = _al_open(uconvert_tofilename(filename, tmp), O_RDONLY | O_BINARY);
 #endif
 
    if (fd < 0) {
@@ -1904,13 +1947,56 @@ PACKFILE *pack_fopen_chunk(PACKFILE *f, int pack)
 
       /* write a sub-chunk */ 
       int tmp_fd = -1;
+      char *tmp_dir = NULL;
+      char *tmp_name = NULL;
+      #ifndef ALLEGRO_HAVE_MKSTEMP
+      char* tmpnam_string;
+      #endif
+
+      #ifdef ALLEGRO_WINDOWS
+         int size;
+         int new_size = 64;
+         
+         /* Get the path of the temporary directory */
+         do {
+            size = new_size;
+            tmp_dir = _AL_REALLOC(tmp_dir, size);
+            new_size = GetTempPath(size, tmp_dir);
+         } while ( (size < new_size) && (new_size > 0) );
+         
+         /* Check if we retrieved the path OK */
+         if (new_size == 0)
+            sprintf(tmp_dir, "%s", "");
+      #else
+         /* Get the path of the temporary directory */
+
+         /* Try various possible locations to store the temporary file */
+         if (getenv("TEMP")) {
+            tmp_dir = strdup(getenv("TEMP"));
+         }
+         else if (getenv("TMP")) {
+            tmp_dir = strdup(getenv("TMP"));
+         }
+         else if (file_exists("/tmp", FA_DIREC, NULL)) {
+            tmp_dir = strdup("/tmp");
+         }
+         else if (getenv("HOME")) {
+            tmp_dir = strdup(getenv("HOME"));
+         }
+         else {
+            /* Give up - try current directory */
+            tmp_dir = strdup(".");
+         }
+
+      #endif
 
       /* the file is open in read/write mode, even if the pack file
        * seems to be in write only mode
        */
-      #ifdef HAVE_MKSTEMP
+      #ifdef ALLEGRO_HAVE_MKSTEMP
 
-         char tmp_name[] = "XXXXXX";
+         tmp_name = _AL_MALLOC_ATOMIC(strlen(tmp_dir) + 16);
+         sprintf(tmp_name, "%s/XXXXXX", tmp_dir);
          tmp_fd = mkstemp(tmp_name);
 
       #else
@@ -1918,7 +2004,10 @@ PACKFILE *pack_fopen_chunk(PACKFILE *f, int pack)
          /* note: since the filename creation and the opening are not
           * an atomic operation, this is not secure
           */
-         char *tmp_name = tmpnam(NULL);
+         tmpnam_string = tmpnam(NULL);
+         tmp_name = _AL_MALLOC_ATOMIC(strlen(tmp_dir) + strlen(tmpnam_string) + 2);
+         sprintf(tmp_name, "%s/%s", tmp_dir, tmpnam_string);
+
          if (tmp_name) {
 #ifndef ALLEGRO_MPW
             tmp_fd = open(tmp_name, O_RDWR | O_BINARY | O_CREAT | O_EXCL, OPEN_PERMS);
@@ -1929,8 +2018,12 @@ PACKFILE *pack_fopen_chunk(PACKFILE *f, int pack)
 
       #endif
 
-      if (tmp_fd < 0)
+      if (tmp_fd < 0) {
+         _AL_FREE(tmp_dir);
+         _AL_FREE(tmp_name);
+      
          return NULL;
+      }
 
       name = uconvert_ascii(tmp_name, tmp);
       chunk = _pack_fdopen(tmp_fd, (pack ? F_WRITE_PACKED : F_WRITE_NOPACK));
@@ -1945,6 +2038,9 @@ PACKFILE *pack_fopen_chunk(PACKFILE *f, int pack)
 
 	 chunk->normal.flags |= PACKFILE_FLAG_CHUNK;
       }
+      
+      _AL_FREE(tmp_dir);
+      _AL_FREE(tmp_name);
    }
    else {
       /* read a sub-chunk */
@@ -1960,9 +2056,9 @@ PACKFILE *pack_fopen_chunk(PACKFILE *f, int pack)
       if (f->normal.flags & PACKFILE_FLAG_OLD_CRYPT) {
 	 /* backward compatibility mode */
 	 if (f->normal.passdata) {
-	    if ((chunk->normal.passdata = malloc(strlen(f->normal.passdata)+1)) == NULL) {
+	    if ((chunk->normal.passdata = _AL_MALLOC_ATOMIC(strlen(f->normal.passdata)+1)) == NULL) {
 	       *allegro_errno = ENOMEM;
-	       free(chunk);
+	       _AL_FREE(chunk);
 	       return NULL;
 	    }
 	    _al_sane_strncpy(chunk->normal.passdata, f->normal.passdata, strlen(f->normal.passdata)+1);
@@ -2076,7 +2172,7 @@ PACKFILE *pack_fclose_chunk(PACKFILE *f)
       pack_fclose(tmp);
 
       delete_file(name);
-      free(name);
+      _AL_FREE(name);
    }
    else {
       /* finish reading a chunk */
@@ -2464,7 +2560,7 @@ int pack_fputs(AL_CONST char *p, PACKFILE *f)
    *allegro_errno = 0;
 
    bufsize = uconvert_size(p, U_CURRENT, U_UTF8);
-   buf = malloc(bufsize);
+   buf = _AL_MALLOC_ATOMIC(bufsize);
    if (!buf)
       return -1;
 
@@ -2480,7 +2576,7 @@ int pack_fputs(AL_CONST char *p, PACKFILE *f)
       s++;
    }
 
-   free(buf);
+   _AL_FREE(buf);
 
    if (*allegro_errno)
       return -1;
@@ -2581,7 +2677,7 @@ static int normal_fclose(void *_f)
    }
 
    if (f->normal.passdata) {
-      free(f->normal.passdata);
+      _AL_FREE(f->normal.passdata);
       f->normal.passdata = NULL;
       f->normal.passpos = NULL;
    }
@@ -2591,6 +2687,28 @@ static int normal_fclose(void *_f)
 
 
 
+/* normal_no_more_input:
+ *  Return non-zero if the number of bytes remaining in the file (todo) is
+ *  less than or equal to zero.
+ *
+ *  However, there is a special case.  If we are reading from a LZSS
+ *  compressed file, the latest call to lzss_read() may have suspended while
+ *  writing out a sequence of bytes due to the output buffer being too small.
+ *  In that case the `todo' count would be decremented (possibly to zero),
+ *  but the output isn't yet completely written out.
+ */
+static INLINE int normal_no_more_input(PACKFILE *f)
+{
+   /* see normal_refill_buffer() to see when lzss_read() is called */
+   if (f->normal.parent && (f->normal.flags & PACKFILE_FLAG_PACK) &&
+       _al_lzss_incomplete_state(f->normal.unpack_data))
+      return 0;
+
+   return (f->normal.todo <= 0);
+}
+
+
+
 static int normal_getc(void *_f)
 {
    PACKFILE *f = _f;
@@ -2600,7 +2718,7 @@ static int normal_getc(void *_f)
       return *(f->normal.buf_pos++);
 
    if (f->normal.buf_size == 0) {
-      if (f->normal.todo <= 0)
+      if (normal_no_more_input(f))
 	 f->normal.flags |= PACKFILE_FLAG_EOF;
       return *(f->normal.buf_pos++);
    }
@@ -2693,7 +2811,7 @@ static int normal_fseek(void *_f, int offset)
       f->normal.buf_size -= i;
       f->normal.buf_pos += i;
       offset -= i;
-      if ((f->normal.buf_size <= 0) && (f->normal.todo <= 0))
+      if ((f->normal.buf_size <= 0) && normal_no_more_input(f))
 	 f->normal.flags |= PACKFILE_FLAG_EOF;
    }
 
@@ -2718,7 +2836,7 @@ static int normal_fseek(void *_f, int offset)
 	    lseek(f->normal.hndl, i, SEEK_CUR);
 	 }
 	 f->normal.todo -= i;
-	 if (f->normal.todo <= 0)
+	 if (normal_no_more_input(f))
 	    f->normal.flags |= PACKFILE_FLAG_EOF;
       }
    }
@@ -2760,7 +2878,7 @@ static int normal_refill_buffer(PACKFILE *f)
    if (f->normal.flags & PACKFILE_FLAG_EOF)
       return EOF;
 
-   if (f->normal.todo <= 0) {
+   if (normal_no_more_input(f)) {
       f->normal.flags |= PACKFILE_FLAG_EOF;
       return EOF;
    }
@@ -2811,7 +2929,7 @@ static int normal_refill_buffer(PACKFILE *f)
    f->normal.buf_pos = f->normal.buf;
    f->normal.buf_size--;
    if (f->normal.buf_size <= 0)
-      if (f->normal.todo <= 0)
+      if (normal_no_more_input(f))
 	 f->normal.flags |= PACKFILE_FLAG_EOF;
 
    if (f->normal.buf_size < 0)
