@@ -193,7 +193,7 @@ static void fli_skip(int bytes)
 #define READ_CHAR_NC(p)    (*(signed char *)(p)++)
 #endif
 
-#if (defined ALLEGRO_GCC) && (defined ALLEGRO_LITTLE_ENDIAN)
+#if (defined ALLEGRO_GCC) && (defined ALLEGRO_LITTLE_ENDIAN) && (!defined ALLEGRO_ARM)
 
 #if 0
 /* the "cast expression as lvalue" extension is deprecated in GCC 3.4 */
@@ -993,7 +993,7 @@ int open_fli(AL_CONST char *filename)
       return FLI_ERROR;
 
    if (fli_filename) {
-      free(fli_filename);
+      _AL_FREE(fli_filename);
       fli_filename = NULL;
    }
 
@@ -1042,7 +1042,7 @@ void close_fli(void)
    }
 
    if (fli_filename) {
-      free(fli_filename);
+      _AL_FREE(fli_filename);
       fli_filename = NULL;
    }
 
