@@ -566,14 +566,14 @@ static void mono_destroy(FONT* f)
         FONT_MONO_DATA* next = mf->next;
         int i = 0;
 
-        for(i = mf->begin; i < mf->end; i++) free(mf->glyphs[i - mf->begin]);
+        for(i = mf->begin; i < mf->end; i++) _AL_FREE(mf->glyphs[i - mf->begin]);
 
-        free(mf->glyphs);
-        free(mf);
+        _AL_FREE(mf->glyphs);
+        _AL_FREE(mf);
         mf = next;
     }
 
-    free(f);
+    _AL_FREE(f);
 }
 
 
@@ -680,7 +680,7 @@ static FONT_MONO_DATA *mono_copy_glyph_range(FONT_MONO_DATA *mf, int begin, int
    if (begin<mf->begin || end>mf->end)
       return NULL;
    
-   newmf = _al_malloc(sizeof *newmf);
+   newmf = _AL_MALLOC(sizeof *newmf);
 
    if (!newmf)
       return NULL;
@@ -690,13 +690,13 @@ static FONT_MONO_DATA *mono_copy_glyph_range(FONT_MONO_DATA *mf, int begin, int
    newmf->next = NULL;
    num = end - begin;
 
-   gl = newmf->glyphs = _al_malloc(num * sizeof *gl);
+   gl = newmf->glyphs = _AL_MALLOC(num * sizeof *gl);
    for (c=0; c<num; c++) {
       int sz;
       g = mf->glyphs[begin - mf->begin + c];
       sz = ((g->w + 7) / 8) * g->h;
 
-      gl[c] = _al_malloc(sz + sizeof *(gl[c]));
+      gl[c] = _AL_MALLOC(sz + sizeof *(gl[c]));
       gl[c]->w = g->w;
       gl[c]->h = g->h;
       memcpy(gl[c]->dat, g->dat, sz * sizeof *(g->dat));
@@ -737,7 +737,7 @@ static FONT *mono_extract_font_range(FONT *f, int begin, int end)
    }
 
    /* Get output font */
-   fontout = _al_malloc(sizeof *fontout);
+   fontout = _AL_MALLOC(sizeof *fontout);
 
    fontout->height = f->height;
    fontout->vtable = f->vtable;
@@ -794,7 +794,7 @@ static FONT *mono_merge_fonts(FONT *font1, FONT *font2)
       return NULL;
 
    /* Get output font */
-   fontout = _al_malloc(sizeof *fontout);
+   fontout = _AL_MALLOC(sizeof *fontout);
    fontout->height = MAX(font1->height, font2->height);
    fontout->vtable = font1->vtable;
    mf = fontout->data = NULL;
@@ -952,6 +952,38 @@ static int color_render_char(AL_CONST FONT* f, int ch, int fg, int bg, BITMAP* b
 
 
 
+/* trans_render_char:
+ *  (trans vtable entry)
+ *
+ *  Renders a transparent character onto a bitmap, at the specified location,
+ *  using the specified colors. fg is ignored. if bg == -1, render as
+ *  transparent, else render as opaque. Returns the character width, in pixels.
+ */
+static int trans_render_char(AL_CONST FONT* f, int ch, int fg, int bg, BITMAP* bmp, int x, int y)
+{
+    int w = 0;
+    int h = f->vtable->font_height(f);
+    BITMAP *g = 0;
+
+    acquire_bitmap(bmp);
+
+    if(bg >= 0) {
+       rectfill(bmp, x, y, x + f->vtable->char_length(f, ch) - 1, y + h - 1, bg);
+    }
+
+    g = _color_find_glyph(f, ch);
+    if(g) {
+       draw_trans_sprite(bmp, g, x, y + (h-g->h)/2);
+       w = g->w;
+    }
+
+    release_bitmap(bmp);
+
+    return w;
+}
+
+
+
 /* color_render:
  *  (color vtable entry)
  *  Renders a color font onto a bitmap, at the specified location, using
@@ -997,13 +1029,13 @@ static void color_destroy(FONT* f)
 
         for(i = cf->begin; i < cf->end; i++) destroy_bitmap(cf->bitmaps[i - cf->begin]);
 
-        free(cf->bitmaps);
-        free(cf);
+        _AL_FREE(cf->bitmaps);
+        _AL_FREE(cf);
 
         cf = next;
     }
 
-    free(f);
+    _AL_FREE(f);
 }
 
 
@@ -1102,8 +1134,8 @@ static int color_get_font_range_end(FONT* f, int range)
  */
 static FONT_COLOR_DATA* upgrade_to_color_data(FONT_MONO_DATA* mf)
 {
-    FONT_COLOR_DATA* cf = _al_malloc(sizeof *cf);
-    BITMAP** bits = _al_malloc((mf->end - mf->begin)*sizeof *bits);
+    FONT_COLOR_DATA* cf = _AL_MALLOC(sizeof *cf);
+    BITMAP** bits = _AL_MALLOC((mf->end - mf->begin)*sizeof *bits);
     int i;
 
     cf->begin = mf->begin;
@@ -1132,7 +1164,7 @@ static FONT *upgrade_to_color(FONT* f)
     FONT *outf;
 
     if (is_color_font(f)) return NULL;
-    outf = _al_malloc(sizeof *outf);
+    outf = _AL_MALLOC(sizeof *outf);
     outf->vtable = font_vtable_color;
     outf->height = f->height;
 
@@ -1165,7 +1197,7 @@ static FONT_COLOR_DATA *color_copy_glyph_range(FONT_COLOR_DATA *cf, int begin, i
    if (begin<cf->begin || end>cf->end)
       return NULL;
    
-   newcf = _al_malloc(sizeof *newcf);
+   newcf = _AL_MALLOC(sizeof *newcf);
 
    if (!newcf)
       return NULL;
@@ -1175,7 +1207,7 @@ static FONT_COLOR_DATA *color_copy_glyph_range(FONT_COLOR_DATA *cf, int begin, i
    newcf->next = NULL;
    num = end - begin;
 
-   gl = newcf->bitmaps = _al_malloc(num * sizeof *gl);
+   gl = newcf->bitmaps = _AL_MALLOC(num * sizeof *gl);
    for (c=0; c<num; c++) {
       g = cf->bitmaps[begin - cf->begin + c];
       gl[c] = create_bitmap_ex(bitmap_color_depth(g), g->w, g->h);
@@ -1217,7 +1249,7 @@ static FONT *color_extract_font_range(FONT *f, int begin, int end)
    }
 
    /* Get output font */
-   fontout = _al_malloc(sizeof *fontout);
+   fontout = _AL_MALLOC(sizeof *fontout);
 
    fontout->height = f->height;
    fontout->vtable = f->vtable;
@@ -1287,7 +1319,7 @@ static FONT *color_merge_fonts(FONT *font1, FONT *font2)
       return NULL;
 
    /* Get output font */
-   fontout = _al_malloc(sizeof *fontout);
+   fontout = _AL_MALLOC(sizeof *fontout);
    fontout->height = MAX(font1->height, font2->height);
    fontout->vtable = font1->vtable;
    cf = fontout->data = NULL;
@@ -1393,6 +1425,77 @@ FONT_VTABLE _font_vtable_color = {
 
 FONT_VTABLE* font_vtable_color = &_font_vtable_color;
 
+FONT_VTABLE _font_vtable_trans = {  
+    font_height,
+    color_char_length,
+    length,
+    trans_render_char,
+    color_render,
+    color_destroy,
+
+    color_get_font_ranges,
+    color_get_font_range_begin,
+    color_get_font_range_end,
+    color_extract_font_range,
+    color_merge_fonts,
+    color_transpose_font
+};
+
+FONT_VTABLE* font_vtable_trans = &_font_vtable_trans;
+
+
+
+/* font_has_alpha:
+ *  Returns TRUE if a color font has an alpha channel.
+ */
+int font_has_alpha(FONT *fnt)
+{
+   FONT_COLOR_DATA *data;
+   int ch;
+
+   ASSERT(fnt);
+
+   if (!is_color_font(fnt))
+      return FALSE;
+
+   data = (FONT_COLOR_DATA *)(fnt->data);
+
+   while (data) {
+      for (ch = data->begin; ch != data->end; ++ch)
+         if (_bitmap_has_alpha(data->bitmaps[ch - data->begin]))
+            return TRUE;
+      data = data->next;
+   }
+
+   return FALSE;
+}
+
+
+
+/* make_trans_font:
+ *  Modifes a font so glyphs are drawn with draw_trans_sprite.
+ */
+void make_trans_font(FONT *f)
+{
+   ASSERT(f);
+   ASSERT(f->vtable == font_vtable_color);
+
+   f->vtable = font_vtable_trans;
+}
+
+
+
+/* is_trans_font:
+ *  Returns non-zero if the font passed is a bitmapped colour font using
+ *  draw_trans_sprite to render glyphs.
+ */
+int is_trans_font(FONT *f)
+{
+   ASSERT(f);
+
+   return (f->vtable == font_vtable_trans);
+}
+
 
 
 /* is_color_font:
@@ -1402,7 +1505,7 @@ int is_color_font(FONT *f)
 {
    ASSERT(f);
    
-   return f->vtable == font_vtable_color;
+   return (f->vtable == font_vtable_color || f->vtable == font_vtable_trans);
 }
 
 
