@@ -21,6 +21,8 @@
 #include "allegro.h"
 #include "allegro/internal/aintern.h"
 
+extern void blit_end(void);   /* for LOCK_FUNCTION; defined in blit.c */
+
 
 
 #define PREFIX_I                "al-gfx INFO: "
@@ -96,10 +98,10 @@ int _rgb_a_shift_32 = DEFAULT_RGB_A_SHIFT_32;
 /* lookup table for scaling 5 bit colors up to 8 bits */
 int _rgb_scale_5[32] =
 {
-   0,   8,   16,  24,  32,  41,  49,  57,
-   65,  74,  82,  90,  98,  106, 115, 123,
-   131, 139, 148, 156, 164, 172, 180, 189,
-   197, 205, 213, 222, 230, 238, 246, 255
+   0,   8,   16,  24,  33,  41,  49,  57,
+   66,  74,  82,  90,  99,  107, 115, 123,
+   132, 140, 148, 156, 165, 173, 181, 189,
+   198, 206, 214, 222, 231, 239, 247, 255
 };
 
 
@@ -108,12 +110,12 @@ int _rgb_scale_6[64] =
 {
    0,   4,   8,   12,  16,  20,  24,  28,
    32,  36,  40,  44,  48,  52,  56,  60,
-   64,  68,  72,  76,  80,  85,  89,  93,
+   65,  69,  73,  77,  81,  85,  89,  93,
    97,  101, 105, 109, 113, 117, 121, 125,
-   129, 133, 137, 141, 145, 149, 153, 157,
-   161, 165, 170, 174, 178, 182, 186, 190,
-   194, 198, 202, 206, 210, 214, 218, 222,
-   226, 230, 234, 238, 242, 246, 250, 255
+   130, 134, 138, 142, 146, 150, 154, 158,
+   162, 166, 170, 174, 178, 182, 186, 190,
+   195, 199, 203, 207, 211, 215, 219, 223,
+   227, 231, 235, 239, 243, 247, 251, 255
 };
 
 
@@ -285,9 +287,9 @@ void destroy_gfx_mode_list(GFX_MODE_LIST *gfx_mode_list)
 {
    if (gfx_mode_list) {
       if (gfx_mode_list->mode)
-         free(gfx_mode_list->mode);
+         _AL_FREE(gfx_mode_list->mode);
 
-      free(gfx_mode_list);
+      _AL_FREE(gfx_mode_list);
    }
 }
 
@@ -624,13 +626,13 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
    char tmp1[64], tmp2[64];
    AL_CONST char *dv;
    int flags = 0;
-   int c, driver, ret;
+   int c;
    ASSERT(system_driver);
    ASSERT(card != GFX_SAFE);
 
    /* remember the current console state */
    if (gfx_virgin) {
-      TRACE(PREFIX_I "Firt call, remembering console state.\n");
+      TRACE(PREFIX_I "First call, remembering console state.\n");
       LOCK_FUNCTION(_stub_bank_switch);
       LOCK_FUNCTION(blit);
 
@@ -709,7 +711,7 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
 	 system_driver->restore_console_state();
 
       if (_gfx_bank) {
-	 free(_gfx_bank);
+	 _AL_FREE(_gfx_bank);
 	 _gfx_bank = NULL;
       }
 
@@ -743,6 +745,8 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
       /* autodetect the driver */
       int found = FALSE;
 
+      tmp1[0] = '\0';
+
       /* first try the config variables */
       if (allow_config) {
 	 /* try the gfx_card variable if GFX_AUTODETECT or GFX_AUTODETECT_FULLSCREEN was selected */
@@ -756,7 +760,7 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
 
       /* go through the list of autodetected drivers if none was previously found */
       if (!found) {
-	 TRACE(PREFIX_I "Autodetecing graphic driver.\n");
+	 TRACE(PREFIX_I "Autodetecting graphic driver.\n");
 	 for (c=0; driver_list[c].driver; c++) {
 	    if (driver_list[c].autodetect) {
 	       drv = driver_list[c].driver;
@@ -771,7 +775,7 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
 	 }
       }
       else {
-	 TRACE(PREFIX_I "GFX_AUTODETECT overriden through configuration:"
+	 TRACE(PREFIX_I "GFX_AUTODETECT overridden through configuration:"
 	       " %s.\n", tmp1);
       }
    }
@@ -793,7 +797,7 @@ static int _set_gfx_mode(int card, int w, int h, int v_w, int v_h, int allow_con
       if (system_driver->display_switch_lock)
 	 system_driver->display_switch_lock(FALSE, FALSE);
 
-      TRACE(PREFIX_E "Failed setting graphic driver %d.", card);
+      TRACE(PREFIX_E "Failed setting graphic driver %d.\n", card);
       return -1;
    }
 
@@ -870,7 +874,6 @@ static int _set_gfx_mode_safe(int card, int w, int h, int v_w, int v_h)
 {
    char buf[ALLEGRO_ERROR_SIZE], tmp1[64];
    struct GFX_MODE mode;
-   static int allow_config = TRUE;
    int ret, driver;
 
    ASSERT(card == GFX_SAFE);
@@ -972,13 +975,13 @@ BITMAP *_make_bitmap(int w, int h, uintptr_t addr, GFX_DRIVER *driver, int color
 
    size = sizeof(BITMAP) + sizeof(char *) * h;
 
-   b = (BITMAP *)malloc(size);
+   b = (BITMAP *)_AL_MALLOC(size);
    if (!b)
       return NULL;
 
-   _gfx_bank = realloc(_gfx_bank, h * sizeof(int));
+   _gfx_bank = _AL_REALLOC(_gfx_bank, h * sizeof(int));
    if (!_gfx_bank) {
-      free(b);
+      _AL_FREE(b);
       return NULL;
    }
 
@@ -1041,7 +1044,10 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
 {
    GFX_VTABLE *vtable;
    BITMAP *bitmap;
+   int nr_pointers;
+   int padding;
    int i;
+
    ASSERT(width >= 0);
    ASSERT(height > 0);
    ASSERT(system_driver);
@@ -1053,13 +1059,23 @@ BITMAP *create_bitmap_ex(int color_depth, int width, int height)
    if (!vtable)
       return NULL;
 
-   bitmap = malloc(sizeof(BITMAP) + (sizeof(char *) * height));
+   /* We need at least two pointers when drawing, otherwise we get crashes with
+    * Electric Fence.  We think some of the assembly code assumes a second line
+    * pointer is always available.
+    */
+   nr_pointers = MAX(2, height);
+   bitmap = _AL_MALLOC(sizeof(BITMAP) + (sizeof(char *) * nr_pointers));
    if (!bitmap)
       return NULL;
 
-   bitmap->dat = malloc(width * height * BYTES_PER_PIXEL(color_depth));
+   /* This avoids a crash for assembler code accessing the last pixel, as it
+    * read 4 bytes instead of 3.
+    */
+   padding = (color_depth == 24) ? 1 : 0;
+
+   bitmap->dat = _AL_MALLOC_ATOMIC(width * height * BYTES_PER_PIXEL(color_depth) + padding);
    if (!bitmap->dat) {
-      free(bitmap);
+      _AL_FREE(bitmap);
       return NULL;
    }
 
@@ -1112,6 +1128,7 @@ BITMAP *create_bitmap(int width, int height)
 BITMAP *create_sub_bitmap(BITMAP *parent, int x, int y, int width, int height)
 {
    BITMAP *bitmap;
+   int nr_pointers;
    int i;
 
    ASSERT(parent);
@@ -1132,7 +1149,9 @@ BITMAP *create_sub_bitmap(BITMAP *parent, int x, int y, int width, int height)
       return system_driver->create_sub_bitmap(parent, x, y, width, height);
 
    /* get memory for structure and line pointers */
-   bitmap = malloc(sizeof(BITMAP) + (sizeof(char *) * height));
+   /* (see create_bitmap for the reason we need at least two) */
+   nr_pointers = MAX(2, height);
+   bitmap = _AL_MALLOC(sizeof(BITMAP) + (sizeof(char *) * nr_pointers));
    if (!bitmap)
       return NULL;
 
@@ -1204,7 +1223,7 @@ static BITMAP *add_vram_block(int x, int y, int w, int h)
    VRAM_BITMAP *b, *new_b;
    VRAM_BITMAP **last_p;
 
-   new_b = malloc(sizeof(VRAM_BITMAP));
+   new_b = _AL_MALLOC(sizeof(VRAM_BITMAP));
    if (!new_b)
       return NULL;
 
@@ -1215,7 +1234,7 @@ static BITMAP *add_vram_block(int x, int y, int w, int h)
 
    new_b->bmp = create_sub_bitmap(screen, x, y, w, h);
    if (!new_b->bmp) {
-      free(new_b);
+      _AL_FREE(new_b);
       return NULL;
    }
 
@@ -1281,7 +1300,7 @@ BITMAP *create_video_bitmap(int width, int height)
       if (!bmp)
 	 return NULL;
 
-      b = malloc(sizeof(VRAM_BITMAP));
+      b = _AL_MALLOC(sizeof(VRAM_BITMAP));
       b->x = -1;
       b->y = -1;
       b->w = 0;
@@ -1437,7 +1456,7 @@ void destroy_bitmap(BITMAP *bitmap)
 	       if (pos->x < 0) {
 		  /* the driver is in charge of this object */
 		  gfx_driver->destroy_video_bitmap(bitmap);
-		  free(pos);
+		  _AL_FREE(pos);
 		  return;
 	       } 
 
@@ -1453,7 +1472,7 @@ void destroy_bitmap(BITMAP *bitmap)
 	       if (failed_bitmap_h > BMP_MAX_SIZE)
 		  failed_bitmap_h = BMP_MAX_SIZE;
 
-	       free(pos);
+	       _AL_FREE(pos);
 	       break;
 	    }
 
@@ -1480,9 +1499,9 @@ void destroy_bitmap(BITMAP *bitmap)
       }
 
       if (bitmap->dat)
-	 free(bitmap->dat);
+	 _AL_FREE(bitmap->dat);
 
-      free(bitmap);
+      _AL_FREE(bitmap);
    }
 }
 
