@@ -1,6 +1,6 @@
-/*         ______   ___    ___ 
- *        /\  _  \ /\_ \  /\_ \ 
- *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
+/*         ______   ___    ___
+ *        /\  _  \ /\_ \  /\_ \
+ *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
  *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
  *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
  *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
@@ -18,6 +18,8 @@
  *
  *      Eric Botcazou added the support for non-blocking menus.
  *
+ *      Elias Pschernig and Sven Sandberg improved the focus algorithm.
+ *
  *      See readme.txt for copyright information.
  */
 
@@ -37,7 +39,17 @@ int gui_mouse_focus = TRUE;
 int gui_font_baseline = 0;
 
 
-/* pointer to the currently active dialog and menu objects */
+/* Pointers to the currently active dialog and menu objects.
+ *
+ * Note: active_dialog_player always points to the currently active dialog
+ * player. However, active_menu_player only ever points to menu players
+ * started by a d_menu_proc. The code also assumes that that d_menu_proc can
+ * be found in the currently active dialog.
+ *
+ * Note: active_dialog points to the whole dialog currently running. However,
+ * active_menu points to the *current item* of the currently running menu,
+ * and should really have been called active_menu_item.
+ */
 static DIALOG_PLAYER *active_dialog_player = NULL;
 static MENU_PLAYER *active_menu_player = NULL;
 static int active_menu_player_zombie = FALSE;
@@ -45,6 +57,9 @@ DIALOG *active_dialog = NULL;
 MENU *active_menu = NULL;
 
 
+static BITMAP *gui_screen = NULL;
+
+
 /* list of currently active (initialized) dialog players */
 struct al_active_dialog_player {
    DIALOG_PLAYER *player;
@@ -55,6 +70,10 @@ static struct al_active_dialog_player *first_active_dialog_player = 0;
 static struct al_active_dialog_player *current_active_dialog_player = 0;
 
 
+/* forward declarations */
+static int shutdown_single_menu(MENU_PLAYER *, int *);
+
+
 
 /* hook function for reading the mouse x position */
 static int default_mouse_x(void)
@@ -146,7 +165,7 @@ static int gui_install_time = 0;
 static void dclick_check(void)
 {
    gui_timer++;
-   
+
    if (dclick_status==DCLICK_START) {              /* first click... */
       if (!gui_mouse_b()) {
 	 dclick_status = DCLICK_RELEASE;           /* aah! released first */
@@ -197,10 +216,10 @@ void position_dialog(DIALOG *dialog, int x, int y)
 
    /* locate the upper-left corner */
    for (c=0; dialog[c].proc; c++) {
-      if (dialog[c].x < min_x) 
+      if (dialog[c].x < min_x)
 	 min_x = dialog[c].x;
 
-      if (dialog[c].y < min_y) 
+      if (dialog[c].y < min_y)
 	 min_y = dialog[c].y;
    }
 
@@ -232,7 +251,7 @@ void centre_dialog(DIALOG *dialog)
 
    /* find the extents of the dialog (done in many loops due to a bug
     * in MSVC that prevents the more sensible version from working)
-    */ 
+    */
    for (c=0; dialog[c].proc; c++) {
       if (dialog[c].x < min_x)
 	 min_x = dialog[c].x;
@@ -350,16 +369,30 @@ int object_message(DIALOG *dialog, int msg, int c)
 
 /* dialog_message:
  *  Sends a message to all the objects in a dialog. If any of the objects
- *  return values other than D_O_K, returns the value and sets obj to the 
+ *  return values other than D_O_K, returns the value and sets obj to the
  *  object which produced it.
  */
 int dialog_message(DIALOG *dialog, int msg, int c, int *obj)
 {
    int count, res, r, force, try;
+   DIALOG *menu_dialog = NULL;
    ASSERT(dialog);
 
-   if (msg == MSG_DRAW)
-      acquire_screen();
+   /* Note: don't acquire the screen in here.  A deadlock develops in a
+    * situation like this:
+    *
+    * 1. this thread: acquires the screen;
+    * 2. timer thread: wakes up, locks the timer_mutex, and calls a
+    *    callback to redraw the software mouse cursor, which tries to
+    *    acquire the screen;
+    * 3. this thread: object_message(MSG_DRAW) calls scare_mouse()
+    *    which calls remove_int().
+    *
+    * So this thread has the screen acquired and wants the timer_mutex,
+    * whereas the timer thread holds the timer_mutex, but wants to acquire
+    * the screen.  The problem is calling scare_mouse() with the screen
+    * acquired.
+    */
 
    force = ((msg == MSG_START) || (msg == MSG_END) || (msg >= MSG_USER));
 
@@ -371,14 +404,16 @@ int dialog_message(DIALOG *dialog, int msg, int c, int *obj)
     * first to send the message to the d_menu_proc object and, if the menu
     * is then not active anymore, send it to the other objects as well.
     */
-   if (active_menu_player)
+   if (active_menu_player) {
       try = 2;
+      menu_dialog = active_menu_player->dialog;
+   }
    else
       try = 1;
 
    for (; try > 0; try--) {
       for (count=0; dialog[count].proc; count++) {
-         if ((try == 2) && (&dialog[count] != active_menu_player->dialog))
+         if ((try == 2) && (&dialog[count] != menu_dialog))
 	    continue;
 
 	 if ((force) || (!(dialog[count].flags & D_HIDDEN))) {
@@ -401,9 +436,6 @@ int dialog_message(DIALOG *dialog, int msg, int c, int *obj)
 	 break;
    }
 
-   if (msg == MSG_DRAW)
-      release_screen();
-
    return res;
 }
 
@@ -433,13 +465,20 @@ static int find_mouse_object(DIALOG *d)
 {
    int mouse_object = -1;
    int c;
+   int res;
    ASSERT(d);
 
-   for (c=0; d[c].proc; c++)
+   for (c=0; d[c].proc; c++) {
       if ((gui_mouse_x() >= d[c].x) && (gui_mouse_y() >= d[c].y) &&
-	  (gui_mouse_x() < d[c].x + d[c].w) && (gui_mouse_y() < d[c].y + d[c].h) &&
-	  (!(d[c].flags & (D_HIDDEN | D_DISABLED))))
-	 mouse_object = c;
+         (gui_mouse_x() < d[c].x + d[c].w) && (gui_mouse_y() < d[c].y + d[c].h) &&
+         (!(d[c].flags & (D_HIDDEN | D_DISABLED)))) {
+         /* check if this object wants the mouse */
+         res = object_message(d+c, MSG_WANTMOUSE, 0);
+         if (!(res & D_DONTWANTMOUSE)) {
+            mouse_object = c;
+         }
+      }
+   }
 
    return mouse_object;
 }
@@ -455,7 +494,7 @@ int offer_focus(DIALOG *dialog, int obj, int *focus_obj, int force)
    ASSERT(dialog);
    ASSERT(focus_obj);
 
-   if ((obj == *focus_obj) || 
+   if ((obj == *focus_obj) ||
        ((obj >= 0) && (dialog[obj].flags & (D_HIDDEN | D_DISABLED))))
       return D_O_K;
 
@@ -506,6 +545,16 @@ typedef struct OBJ_LIST
 } OBJ_LIST;
 
 
+/* Weight ratio between the orthogonal direction and the main direction
+   when calculating the distance for the focus algorithm. */
+#define DISTANCE_RATIO  8
+
+/* Maximum size (in bytes) of a dialog array. */
+#define MAX_SIZE  0x10000  /* 64 kb */
+
+enum axis { X_AXIS, Y_AXIS };
+
+
 
 /* obj_list_cmp:
  *  Callback function for qsort().
@@ -524,8 +573,9 @@ static int cmp_tab(AL_CONST DIALOG *d1, AL_CONST DIALOG *d2)
 {
    int ret = (int)((AL_CONST unsigned long)d2 - (AL_CONST unsigned long)d1);
 
+   /* Wrap around if d2 is before d1 in the dialog array. */
    if (ret < 0)
-      ret += 0x10000;
+      ret += MAX_SIZE;
 
    return ret;
 }
@@ -539,25 +589,83 @@ static int cmp_shift_tab(AL_CONST DIALOG *d1, AL_CONST DIALOG *d2)
 {
    int ret = (int)((AL_CONST unsigned long)d1 - (AL_CONST unsigned long)d2);
 
+   /* Wrap around if d2 is after d1 in the dialog array. */
    if (ret < 0)
-      ret += 0x10000;
+      ret += MAX_SIZE;
 
    return ret;
 }
 
 
 
+/* min_dist:
+ *  Returns the minimum distance between dialogs 'd1' and 'd2'. 'main_axis'
+ *  is taken account to give different weights to the axes in the distance
+ *  formula, as well as to shift the actual position of 'd2' along the axis
+ *  by the amount specified by 'bias'.
+ */
+static int min_dist(AL_CONST DIALOG *d1, AL_CONST DIALOG *d2, enum axis main_axis, int bias)
+{
+   int x_left = d1->x - d2->x - d2->w + 1;
+   int x_right = d2->x - d1->x - d1->w + 1;
+   int y_top = d1->y - d2->y - d2->h + 1;
+   int y_bottom = d2->y - d1->y - d1->h + 1;
+
+   if (main_axis == X_AXIS) {
+      x_left -= bias;
+      x_right += bias;
+      y_top *= DISTANCE_RATIO;
+      y_bottom *= DISTANCE_RATIO;
+   }
+   else {
+      x_left *= DISTANCE_RATIO;
+      x_right *= DISTANCE_RATIO;
+      y_top -= bias;
+      y_bottom += bias;
+   }
+
+   if (x_left > 0) { /* d2 is left of d1 */
+      if (y_top > 0)  /* d2 is above d1 */
+         return x_left + y_top;
+      else if (y_bottom > 0)  /* d2 is below d1 */
+         return x_left + y_bottom;
+      else  /* vertically overlapping */
+         return x_left;
+   }
+   else if (x_right > 0) { /* d2 is right of d1 */
+      if (y_top > 0)  /* d2 is above d1 */
+         return x_right + y_top;
+      else if (y_bottom > 0)  /* d2 is below d1 */
+         return x_right + y_bottom;
+      else  /* vertically overlapping */
+         return x_right;
+   }
+   /* horizontally overlapping */
+   else if (y_top > 0)  /* d2 is above d1 */
+      return y_top;
+   else if (y_bottom > 0)  /* d2 is below d1 */
+      return y_bottom;
+   else  /* overlapping */
+      return 0;
+}
+
+
+
 /* cmp_right:
  *  Comparison function for right arrow key movement.
  */
 static int cmp_right(AL_CONST DIALOG *d1, AL_CONST DIALOG *d2)
 {
-   int ret = (d2->x - d1->x) + ABS(d1->y - d2->y) * 8;
+   int bias;
 
-   if (d1->x >= d2->x)
-      ret += 0x10000;
+   /* Wrap around if d2 is not fully contained in the half-plan
+      delimited by d1's right edge and not containing it. */
+   if (d2->x < d1->x + d1->w)
+      bias = +SCREEN_W;
+   else
+      bias = 0;
 
-   return ret;
+   return min_dist(d1, d2, X_AXIS, bias);
 }
 
 
@@ -567,12 +675,16 @@ static int cmp_right(AL_CONST DIALOG *d1, AL_CONST DIALOG *d2)
  */
 static int cmp_left(AL_CONST DIALOG *d1, AL_CONST DIALOG *d2)
 {
-   int ret = (d1->x - d2->x) + ABS(d1->y - d2->y) * 8;
+   int bias;
 
-   if (d1->x <= d2->x)
-      ret += 0x10000;
+   /* Wrap around if d2 is not fully contained in the half-plan
+      delimited by d1's left edge and not containing it. */
+   if (d2->x + d2->w > d1->x)
+      bias = -SCREEN_W;
+   else
+      bias = 0;
 
-   return ret;
+   return min_dist(d1, d2, X_AXIS, bias);
 }
 
 
@@ -582,12 +694,16 @@ static int cmp_left(AL_CONST DIALOG *d1, AL_CONST DIALOG *d2)
  */
 static int cmp_down(AL_CONST DIALOG *d1, AL_CONST DIALOG *d2)
 {
-   int ret = (d2->y - d1->y) + ABS(d1->x - d2->x) * 8;
+   int bias;
 
-   if (d1->y >= d2->y)
-      ret += 0x10000;
+   /* Wrap around if d2 is not fully contained in the half-plan
+      delimited by d1's bottom edge and not containing it. */
+   if (d2->y < d1->y + d1->h)
+      bias = +SCREEN_H;
+   else
+      bias = 0;
 
-   return ret;
+   return min_dist(d1, d2, Y_AXIS, bias);
 }
 
 
@@ -597,12 +713,16 @@ static int cmp_down(AL_CONST DIALOG *d1, AL_CONST DIALOG *d2)
  */
 static int cmp_up(AL_CONST DIALOG *d1, AL_CONST DIALOG *d2)
 {
-   int ret = (d1->y - d2->y) + ABS(d1->x - d2->x) * 8;
+   int bias;
 
-   if (d1->y <= d2->y)
-      ret += 0x10000;
+   /* Wrap around if d2 is not fully contained in the half-plan
+      delimited by d1's top edge and not containing it. */
+   if (d2->y + d2->h > d1->y)
+      bias = -SCREEN_H;
+   else
+      bias = 0;
 
-   return ret;
+   return min_dist(d1, d2, Y_AXIS, bias);
 }
 
 
@@ -619,7 +739,7 @@ static int move_focus(DIALOG *d, int ascii, int scan, int *focus_obj)
    int fobj, c;
    int res = D_O_K;
 
-   /* choose a comparison function */ 
+   /* choose a comparison function */
    switch (scan) {
       case KEY_TAB:   cmp = (ascii == '\t') ? cmp_tab : cmp_shift_tab; break;
       case KEY_RIGHT: cmp = cmp_right; break;
@@ -631,7 +751,8 @@ static int move_focus(DIALOG *d, int ascii, int scan, int *focus_obj)
 
    /* fill temporary table */
    for (c=0; d[c].proc; c++) {
-      if ((*focus_obj < 0) || (c != *focus_obj)) {
+      if (((*focus_obj < 0) || (c != *focus_obj))
+	  && !(d[c].flags & (D_DISABLED | D_HIDDEN))) {
 	 obj[obj_count].index = c;
 	 if (*focus_obj >= 0)
 	    obj[obj_count].diff = cmp(d+*focus_obj, d+c);
@@ -652,7 +773,7 @@ static int move_focus(DIALOG *d, int ascii, int scan, int *focus_obj)
       res |= offer_focus(d, obj[c].index, focus_obj, FALSE);
       if (fobj != *focus_obj)
 	 break;
-   } 
+   }
 
    return res;
 }
@@ -671,25 +792,31 @@ static int move_focus(DIALOG *d, int ascii, int scan, int *focus_obj)
 
 /* do_dialog:
  *  The basic dialog manager. The list of dialog objects should be
- *  terminated by one with a null dialog procedure. Returns the index of 
+ *  terminated by one with a null dialog procedure. Returns the index of
  *  the object which caused it to exit.
  */
 int do_dialog(DIALOG *dialog, int focus_obj)
 {
    BITMAP *mouse_screen = _mouse_screen;
+   BITMAP *gui_bmp = gui_get_screen();
    int screen_count = _gfx_mode_set_count;
    void *player;
    ASSERT(dialog);
 
-   if (!is_same_bitmap(_mouse_screen, screen))
-      show_mouse(screen);
+   if (!is_same_bitmap(_mouse_screen, gui_bmp) && !(gfx_capabilities&GFX_HW_CURSOR))
+      show_mouse(gui_bmp);
 
    player = init_dialog(dialog, focus_obj);
 
-   while (update_dialog(player))
-      ;
+   while (update_dialog(player)) {
+      /* If a menu is active, we yield here, since the dialog
+       * engine is shut down so no user code can be running.
+       */
+      if (active_menu_player)
+         rest(1);
+   }
 
-   if (_gfx_mode_set_count == screen_count)
+   if (_gfx_mode_set_count == screen_count && !(gfx_capabilities&GFX_HW_CURSOR))
       show_mouse(mouse_screen);
 
    return shutdown_dialog(player);
@@ -706,14 +833,15 @@ int do_dialog(DIALOG *dialog, int focus_obj)
 int popup_dialog(DIALOG *dialog, int focus_obj)
 {
    BITMAP *bmp;
+   BITMAP *gui_bmp;
    int ret;
    ASSERT(dialog);
 
-   bmp = create_bitmap(dialog->w, dialog->h); 
-
+   bmp = create_bitmap(dialog->w, dialog->h);
+   gui_bmp = gui_get_screen();
    if (bmp) {
       scare_mouse_area(dialog->x, dialog->y, dialog->w, dialog->h);
-      blit(screen, bmp, dialog->x, dialog->y, 0, 0, dialog->w, dialog->h);
+      blit(gui_bmp, bmp, dialog->x, dialog->y, 0, 0, dialog->w, dialog->h);
       unscare_mouse();
    }
    else
@@ -723,7 +851,7 @@ int popup_dialog(DIALOG *dialog, int focus_obj)
 
    if (bmp) {
       scare_mouse_area(dialog->x, dialog->y, dialog->w, dialog->h);
-      blit(bmp, screen, 0, 0, dialog->x, dialog->y, dialog->w, dialog->h);
+      blit(bmp, gui_bmp, 0, 0, dialog->x, dialog->y, dialog->w, dialog->h);
       unscare_mouse();
       destroy_bitmap(bmp);
    }
@@ -740,22 +868,27 @@ int popup_dialog(DIALOG *dialog, int focus_obj)
 DIALOG_PLAYER *init_dialog(DIALOG *dialog, int focus_obj)
 {
    DIALOG_PLAYER *player;
+   BITMAP *gui_bmp = gui_get_screen();
    struct al_active_dialog_player *n;
    char tmp1[64], tmp2[64];
    int c;
    ASSERT(dialog);
 
-   player = malloc(sizeof(DIALOG_PLAYER));
+   /* close any menu opened by a d_menu_proc */
+   if (active_menu_player)
+      object_message(active_menu_player->dialog, MSG_LOSTMOUSE, 0);
+
+   player = _AL_MALLOC(sizeof(DIALOG_PLAYER));
    if (!player) {
       *allegro_errno = ENOMEM;
       return NULL;
    }
 
    /* append player to the list */
-   n = malloc(sizeof(struct al_active_dialog_player));
+   n = _AL_MALLOC(sizeof(struct al_active_dialog_player));
    if (!n) {
       *allegro_errno = ENOMEM;
-      free (player);
+      _AL_FREE (player);
       return NULL;
    }
 
@@ -816,7 +949,7 @@ DIALOG_PLAYER *init_dialog(DIALOG *dialog, int focus_obj)
          /* no auto opening */
          gui_menu_opening_delay = -1;
       }
-      
+
       gui_install_count = 1;
       gui_install_time = _allegro_count;
    }
@@ -824,7 +957,8 @@ DIALOG_PLAYER *init_dialog(DIALOG *dialog, int focus_obj)
       gui_install_count++;
 
    /* initialise the dialog */
-   set_clip(screen, 0, 0, SCREEN_W-1, SCREEN_H-1);
+   set_clip_rect(gui_bmp, 0, 0, SCREEN_W-1, SCREEN_H-1);
+   set_clip_state(gui_bmp, TRUE);
    player->res |= dialog_message(dialog, MSG_START, 0, &player->obj);
 
    player->mouse_obj = find_mouse_object(dialog);
@@ -851,6 +985,26 @@ DIALOG_PLAYER *init_dialog(DIALOG *dialog, int focus_obj)
 
 
 
+/* gui_set_screen:
+ *  Changes the target bitmap for GUI drawing operations
+ */
+void gui_set_screen(BITMAP *bmp)
+{
+   gui_screen = bmp;
+}
+
+
+
+/* gui_get_screen:
+ *  Returns the gui_screen, or the default surface if gui_screen is NULL
+ */
+BITMAP *gui_get_screen(void)
+{
+   return gui_screen?gui_screen:screen;
+}
+
+
+
 /* check_for_redraw:
  *  Checks whether any parts of the current dialog need to be redraw.
  */
@@ -895,7 +1049,7 @@ int update_dialog(DIALOG_PLAYER *player)
    int c, cascii, cscan, ccombo, r, ret, nowhere, z;
    int new_mouse_b;
    ASSERT(player);
-   
+
    /* redirect to update_menu() whenever a menu is activated */
    if (active_menu_player) {
       if (!active_menu_player_zombie) {
@@ -914,12 +1068,13 @@ int update_dialog(DIALOG_PLAYER *player)
 	 for (c=0; player->dialog[c].proc; c++)
 	    if (&player->dialog[c] == active_menu_player->dialog)
 	       break;
+	 ASSERT(player->dialog[c].proc);
 
 	 MESSAGE(c, MSG_LOSTMOUSE, 0);
 	 goto getout;
       }
    }
-       
+
    if (player->res & D_CLOSE)
       return FALSE;
 
@@ -948,7 +1103,7 @@ int update_dialog(DIALOG_PLAYER *player)
          player->mouse_b = new_mouse_b;
       }
       else
-	 dialog_message(player->dialog, MSG_IDLE, 0, &nowhere);
+	 player->res |= dialog_message(player->dialog, MSG_IDLE, 0, &nowhere);
    }
 
    /* need to reinstall the dclick and switch handlers? */
@@ -966,13 +1121,13 @@ int update_dialog(DIALOG_PLAYER *player)
 
    /* are we dealing with a mouse click? */
    if (player->click_wait) {
-      if ((ABS(player->mouse_ox-gui_mouse_x()) > 8) || 
+      if ((ABS(player->mouse_ox-gui_mouse_x()) > 8) ||
 	  (ABS(player->mouse_oy-gui_mouse_y()) > 8))
 	 dclick_status = DCLICK_NOT;
 
       /* waiting... */
       if ((dclick_status != DCLICK_AGAIN) && (dclick_status != DCLICK_NOT)) {
-	 dialog_message(player->dialog, MSG_IDLE, 0, &nowhere);
+	 player->res |= dialog_message(player->dialog, MSG_IDLE, 0, &nowhere);
 	 check_for_redraw(player);
 	 return TRUE;
       }
@@ -981,7 +1136,7 @@ int update_dialog(DIALOG_PLAYER *player)
 
       /* double click! */
       if ((dclick_status==DCLICK_AGAIN) &&
-	  (gui_mouse_x() >= player->dialog[player->mouse_obj].x) && 
+	  (gui_mouse_x() >= player->dialog[player->mouse_obj].x) &&
 	  (gui_mouse_y() >= player->dialog[player->mouse_obj].y) &&
 	  (gui_mouse_x() <= player->dialog[player->mouse_obj].x + player->dialog[player->mouse_obj].w) &&
 	  (gui_mouse_y() <= player->dialog[player->mouse_obj].y + player->dialog[player->mouse_obj].h)) {
@@ -1010,7 +1165,7 @@ int update_dialog(DIALOG_PLAYER *player)
 	 player->dialog[c].flags |= D_GOTMOUSE;
 	 MESSAGE(c, MSG_GOTMOUSE, 0);
       }
-      player->mouse_obj = c; 
+      player->mouse_obj = c;
 
       /* move the input focus as well? */
       if ((gui_mouse_focus) && (player->mouse_obj != player->focus_obj))
@@ -1063,7 +1218,7 @@ int update_dialog(DIALOG_PLAYER *player)
    poll_joystick();
 
    if (player->joy_on) {
-      if ((!joy[0].stick[0].axis[0].d1) && (!joy[0].stick[0].axis[0].d2) && 
+      if ((!joy[0].stick[0].axis[0].d1) && (!joy[0].stick[0].axis[0].d2) &&
 	  (!joy[0].stick[0].axis[1].d1) && (!joy[0].stick[0].axis[1].d2) &&
 	  (!joy[0].button[0].b) && (!joy[0].button[1].b)) {
 	 player->joy_on = FALSE;
@@ -1111,19 +1266,19 @@ int update_dialog(DIALOG_PLAYER *player)
       /* let object deal with the key */
       if (player->focus_obj >= 0) {
 	 MESSAGE(player->focus_obj, MSG_CHAR, ccombo);
-	 if (player->res & D_USED_CHAR)
+	 if (player->res & (D_USED_CHAR | D_CLOSE))
 	    goto getout;
 
 	 MESSAGE(player->focus_obj, MSG_UCHAR, cascii);
-	 if (player->res & D_USED_CHAR)
+	 if (player->res & (D_USED_CHAR | D_CLOSE))
 	    goto getout;
       }
 
       /* keyboard shortcut? */
       for (c=0; player->dialog[c].proc; c++) {
-	 if ((((cascii > 0) && (cascii <= 255) && 
+	 if ((((cascii > 0) && (cascii <= 255) &&
 	       (utolower(player->dialog[c].key) == utolower((cascii)))) ||
-	      ((!cascii) && (player->dialog[c].key == (cscan<<8)))) && 
+	      ((!cascii) && (player->dialog[c].key == (cscan<<8)))) &&
 	     (!(player->dialog[c].flags & (D_HIDDEN | D_DISABLED)))) {
 	    MESSAGE(c, MSG_KEY, ccombo);
 	    goto getout;
@@ -1140,7 +1295,7 @@ int update_dialog(DIALOG_PLAYER *player)
       }
 
       /* pass <CR> or <SPACE> to selected object */
-      if (((cascii == '\r') || (cascii == '\n') || (cascii == ' ')) && 
+      if (((cascii == '\r') || (cascii == '\n') || (cascii == ' ')) &&
 	  (player->focus_obj >= 0)) {
 	 MESSAGE(player->focus_obj, MSG_KEY, ccombo);
 	 goto getout;
@@ -1207,7 +1362,7 @@ int shutdown_dialog(DIALOG_PLAYER *player)
 	 if (iter == current_active_dialog_player)
 	    current_active_dialog_player = prev;
 
-	 free (iter);
+	 _AL_FREE (iter);
 	 break;
       }
    }
@@ -1224,7 +1379,7 @@ int shutdown_dialog(DIALOG_PLAYER *player)
 
    obj = player->obj;
 
-   free(player);
+   _AL_FREE(player);
 
    return obj;
 }
@@ -1266,7 +1421,7 @@ static int bar_entry_length(const char *text)
    len = gui_strlen(tok1) + 16;
    if (tok2)
       len += gui_strlen(tok2) + 16;
-   free(buf);
+   _AL_FREE(buf);
 
    return len;
 }
@@ -1307,6 +1462,7 @@ static void draw_menu_item(MENU_PLAYER *m, int c)
    int x, y, w;
    char *buf, *tok1, *tok2;
    int my;
+   BITMAP *gui_bmp = gui_get_screen();
 
    get_menu_pos(m, c, &x, &y, &w);
 
@@ -1324,7 +1480,7 @@ static void draw_menu_item(MENU_PLAYER *m, int c)
       else {
 	 fg = gui_mg_color;
 	 bg = gui_bg_color;
-      } 
+      }
    }
    else {
       if (c == m->sel) {
@@ -1334,38 +1490,38 @@ static void draw_menu_item(MENU_PLAYER *m, int c)
       else {
 	 fg = gui_fg_color;
 	 bg = gui_bg_color;
-      } 
+      }
    }
 
-   rectfill(screen, x, y, x+w-1, y+text_height(font)+3, bg);
+   rectfill(gui_bmp, x, y, x+w-1, y+text_height(font)+3, bg);
 
    if (ugetc(m->menu[c].text)) {
       buf = split_around_tab(m->menu[c].text, &tok1, &tok2);
-      gui_textout_ex(screen, tok1, x+8, y+1, fg, bg, FALSE);
+      gui_textout_ex(gui_bmp, tok1, x+8, y+1, fg, bg, FALSE);
       if (tok2)
- 	 gui_textout_ex(screen, tok2, x+w-gui_strlen(tok2)-10, y+1, fg, bg, FALSE);
+ 	 gui_textout_ex(gui_bmp, tok2, x+w-gui_strlen(tok2)-10, y+1, fg, bg, FALSE);
 
       if ((m->menu[c].child) && (!m->bar)) {
          my = y + text_height(font)/2;
-         hline(screen, x+w-8, my+1, x+w-4, fg);
-         hline(screen, x+w-8, my+0, x+w-5, fg);
-         hline(screen, x+w-8, my-1, x+w-6, fg);
-         hline(screen, x+w-8, my-2, x+w-7, fg);
-         putpixel(screen, x+w-8, my-3, fg);
-         hline(screen, x+w-8, my+2, x+w-5, fg);
-         hline(screen, x+w-8, my+3, x+w-6, fg);
-         hline(screen, x+w-8, my+4, x+w-7, fg);
-         putpixel(screen, x+w-8, my+5, fg);
+         hline(gui_bmp, x+w-8, my+1, x+w-4, fg);
+         hline(gui_bmp, x+w-8, my+0, x+w-5, fg);
+         hline(gui_bmp, x+w-8, my-1, x+w-6, fg);
+         hline(gui_bmp, x+w-8, my-2, x+w-7, fg);
+         putpixel(gui_bmp, x+w-8, my-3, fg);
+         hline(gui_bmp, x+w-8, my+2, x+w-5, fg);
+         hline(gui_bmp, x+w-8, my+3, x+w-6, fg);
+         hline(gui_bmp, x+w-8, my+4, x+w-7, fg);
+         putpixel(gui_bmp, x+w-8, my+5, fg);
       }
 
-      free(buf);
+      _AL_FREE(buf);
    }
    else
-      hline(screen, x, y+text_height(font)/2+2, x+w, fg);
+      hline(gui_bmp, x, y+text_height(font)/2+2, x+w, fg);
 
    if (m->menu[c].flags & D_SELECTED) {
-      line(screen, x+1, y+text_height(font)/2+1, x+3, y+text_height(font)+1, fg);
-      line(screen, x+3, y+text_height(font)+1, x+6, y+2, fg);
+      line(gui_bmp, x+1, y+text_height(font)/2+1, x+3, y+text_height(font)+1, fg);
+      line(gui_bmp, x+3, y+text_height(font)+1, x+6, y+2, fg);
    }
 }
 
@@ -1381,9 +1537,11 @@ static void draw_menu(MENU_PLAYER *m)
    if (gui_menu_draw_menu)
       gui_menu_draw_menu(m->x, m->y, m->w, m->h);
    else {
-      rect(screen, m->x, m->y, m->x+m->w-2, m->y+m->h-2, gui_fg_color);
-      vline(screen, m->x+m->w-1, m->y+1, m->y+m->h-1, gui_fg_color);
-      hline(screen, m->x+1, m->y+m->h-1, m->x+m->w-1, gui_fg_color);
+      BITMAP *gui_bmp = gui_get_screen();
+      rectfill(gui_bmp, m->x, m->y, m->x+m->w-2, m->y+m->h-2, gui_bg_color);
+      rect(gui_bmp, m->x, m->y, m->x+m->w-2, m->y+m->h-2, gui_fg_color);
+      vline(gui_bmp, m->x+m->w-1, m->y+1, m->y+m->h-1, gui_fg_color);
+      hline(gui_bmp, m->x+1, m->y+m->h-1, m->x+m->w-1, gui_fg_color);
    }
 
    for (c=0; m->menu[c].text; c++)
@@ -1431,7 +1589,7 @@ static INLINE int mouse_in_single_menu(MENU_PLAYER *m)
  *  Recursively checks if the mouse is inside a menu (or any of its parents)
  *  and simultaneously not on the selected item of the menu.
  */
-static int mouse_in_parent_menu(MENU_PLAYER *m) 
+static int mouse_in_parent_menu(MENU_PLAYER *m)
 {
    int c;
 
@@ -1494,7 +1652,7 @@ static void layout_menu(MENU_PLAYER *m, MENU *menu, int bar, int x, int y, int m
 	       extra = MAX(extra, c);
 	    }
 
-	    free(buf);
+	    _AL_FREE(buf);
 	 }
       }
    }
@@ -1536,12 +1694,12 @@ static int menu_key_shortcut(int c, AL_CONST char *s)
  *  Searches a menu for keyboard shortcuts, for the alt+letter to bring
  *  up a menu.
  */
-int menu_alt_key(int k, MENU *m)
+static int menu_alt_key(int k, MENU *m)
 {
    static unsigned char alt_table[] =
    {
-      KEY_A, KEY_B, KEY_C, KEY_D, KEY_E, KEY_F, KEY_G, KEY_H, KEY_I, 
-      KEY_J, KEY_K, KEY_L, KEY_M, KEY_N, KEY_O, KEY_P, KEY_Q, KEY_R, 
+      KEY_A, KEY_B, KEY_C, KEY_D, KEY_E, KEY_F, KEY_G, KEY_H, KEY_I,
+      KEY_J, KEY_K, KEY_L, KEY_M, KEY_N, KEY_O, KEY_P, KEY_Q, KEY_R,
       KEY_S, KEY_T, KEY_U, KEY_V, KEY_W, KEY_X, KEY_Y, KEY_Z
    };
 
@@ -1556,7 +1714,8 @@ int menu_alt_key(int k, MENU *m)
    c = scancode_to_ascii(k);
    if (c) {
       k = c;
-   } else {
+   }
+   else {
       for (c=0; c<(int)sizeof(alt_table); c++) {
 	 if (k == alt_table[c]) {
 	    k = c + 'a';
@@ -1598,7 +1757,7 @@ int do_menu(MENU *menu, int x, int y)
    player = init_menu(menu, x ,y);
 
    while (update_menu(player))
-      yield_timeslice();
+      rest(1);
 
    ret = shutdown_menu(player);
 
@@ -1615,10 +1774,12 @@ int do_menu(MENU *menu, int x, int y)
  */
 static MENU_PLAYER *init_single_menu(MENU *menu, MENU_PLAYER *parent, DIALOG *dialog, int bar, int x, int y, int repos, int minw, int minh)
 {
+   BITMAP *gui_bmp = gui_get_screen();
+   int scare = is_same_bitmap(gui_bmp, _mouse_screen);
    MENU_PLAYER *player;
    ASSERT(menu);
 
-   player = malloc(sizeof(MENU_PLAYER));
+   player = _AL_MALLOC(sizeof(MENU_PLAYER));
    if (!player) {
       *allegro_errno = ENOMEM;
       return NULL;
@@ -1631,20 +1792,22 @@ static MENU_PLAYER *init_single_menu(MENU *menu, MENU_PLAYER *parent, DIALOG *di
       player->y = MID(0, player->y, SCREEN_H-player->h-1);
    }
 
-   scare_mouse_area(player->x, player->y, player->w, player->h);
+   if (scare)
+      scare_mouse_area(player->x, player->y, player->w, player->h);
 
    /* save screen under the menu */
-   player->saved = create_bitmap(player->w, player->h); 
+   player->saved = create_bitmap(player->w, player->h);
 
    if (player->saved)
-      blit(screen, player->saved, player->x, player->y, 0, 0, player->w, player->h);
+      blit(gui_bmp, player->saved, player->x, player->y, 0, 0, player->w, player->h);
    else
       *allegro_errno = ENOMEM;
 
    /* setup state variables */
    player->sel = menu_mouse_object(player);
 
-   unscare_mouse();
+   if (scare)
+      unscare_mouse();
 
    player->mouse_button_was_pressed = gui_mouse_b();
    player->back_from_child = FALSE;
@@ -1667,7 +1830,7 @@ static MENU_PLAYER *init_single_menu(MENU *menu, MENU_PLAYER *parent, DIALOG *di
 /* init_menu:
  *  Sets up a menu, returning a menu player object that can be used
  *  with the update_menu() and shutdown_menu() functions.
- */ 
+ */
 MENU_PLAYER *init_menu(MENU *menu, int x, int y)
 {
    return init_single_menu(menu, NULL, NULL, FALSE, x, y, TRUE, 0, 0);
@@ -1687,8 +1850,6 @@ MENU_PLAYER *init_menu(MENU *menu, int x, int y)
  */
 int update_menu(MENU_PLAYER *player)
 {
-   static int shutdown_single_menu(MENU_PLAYER *, int *);
-
    MENU_PLAYER *i;
    int c, c2;
    int old_sel, child_ret;
@@ -1834,10 +1995,13 @@ int update_menu(MENU_PLAYER *player)
    }  /* end of input processing */
 
    if ((player->redraw) || (player->sel != old_sel)) {  /* selection changed? */
+      BITMAP *gui_bmp = gui_get_screen();
+      int scare = is_same_bitmap(gui_bmp, _mouse_screen);
       player->timestamp = gui_timer;
 
-      scare_mouse_area(player->x, player->y, player->w, player->h);
-      acquire_screen();
+      if (scare)
+	 scare_mouse_area(player->x, player->y, player->w, player->h);
+      acquire_bitmap(gui_bmp);
 
       if (player->redraw) {
 	 draw_menu(player);
@@ -1851,8 +2015,9 @@ int update_menu(MENU_PLAYER *player)
 	    draw_menu_item(player, player->sel);
       }
 
-      release_screen();
-      unscare_mouse();
+      release_bitmap(gui_bmp);
+      if (scare)
+	 unscare_mouse();
    }
 
    if (player->auto_open && (gui_menu_opening_delay >= 0)) {  /* menu auto-opening on? */
@@ -1902,16 +2067,16 @@ int update_menu(MENU_PLAYER *player)
 	 player->child = init_single_menu(player->menu[player->ret].child, player, NULL, FALSE, child_x, child_y, TRUE, 0, 0);
 	 return TRUE;  /* continue */
       }
-      
+
       while (player->parent) {  /* parent menu? */
 	 player = player->parent;
 	 shutdown_single_menu(player->child, NULL);
 	 player->child = NULL;
       }
-      
+
       return FALSE;  /* item selected */
    }
-   
+
    if (player->ret < -1) {  /* dismiss menu ? */
       if (player->parent) {
 	 child_ret = player->ret;  /* needed below */
@@ -1928,17 +2093,17 @@ int update_menu(MENU_PLAYER *player)
 	    player->timestamp = gui_timer;
 	    player->back_from_child = TRUE;
 	 }
-	 
+
 	 return TRUE;  /* return to parent */
       }
-      
+
       return FALSE;  /* menu dismissed */
    }
-   
+
    /* special kludge for menu bar */
    if ((player->bar) && (!gui_mouse_b()) && (!keypressed()) && (!mouse_in_single_menu(player)))
       return FALSE;
-   
+
    return TRUE;
 }
 
@@ -1974,16 +2139,20 @@ static int shutdown_single_menu(MENU_PLAYER *player, int *dret)
 
    /* restore screen */
    if (player->saved) {
-      scare_mouse_area(player->x, player->y, player->w, player->h);
-      blit(player->saved, screen, 0, 0, player->x, player->y, player->w, player->h);
-      unscare_mouse();
+      BITMAP *gui_bmp = gui_get_screen();
+      int scare = is_same_bitmap(gui_bmp, _mouse_screen);
+      if (scare)
+         scare_mouse_area(player->x, player->y, player->w, player->h);
+      blit(player->saved, gui_bmp, 0, 0, player->x, player->y, player->w, player->h);
+      if (scare)
+         unscare_mouse();
       destroy_bitmap(player->saved);
    }
 
    ret = player->ret;
-   
-   free(player);
-   
+
+   _AL_FREE(player);
+
    return ret;
 }
 
@@ -2020,7 +2189,7 @@ int shutdown_menu(MENU_PLAYER *player)
 
 
 /* d_menu_proc:
- *  Dialog procedure for adding drop down menus to a GUI dialog. This 
+ *  Dialog procedure for adding drop down menus to a GUI dialog. This
  *  displays the top level menu items as a horizontal bar (eg. across the
  *  top of the screen), and pops up child menus when they are clicked.
  *  When it executes one of the menu callback routines, it passes the
@@ -2120,9 +2289,9 @@ static DIALOG alert_dialog[] =
 
 /* alert3:
  *  Displays a simple alert box, containing three lines of text (s1-s3),
- *  and with either one, two, or three buttons. The text for these buttons 
+ *  and with either one, two, or three buttons. The text for these buttons
  *  is passed in b1, b2, and b3 (NULL for buttons which are not used), and
- *  the keyboard shortcuts in c1 and c2. Returns 1, 2, or 3 depending on 
+ *  the keyboard shortcuts in c1 and c2. Returns 1, 2, or 3 depending on
  *  which button was selected.
  */
 int alert3(AL_CONST char *s1, AL_CONST char *s2, AL_CONST char *s3, AL_CONST char *b1, AL_CONST char *b2, AL_CONST char *b3, int c1, int c2, int c3)
@@ -2154,7 +2323,7 @@ int alert3(AL_CONST char *s1, AL_CONST char *s2, AL_CONST char *s3, AL_CONST cha
    avg_w = text_length(font, tmp);
    avg_h = text_height(font);
 
-   alert_dialog[A_S1].dp = alert_dialog[A_S2].dp = alert_dialog[A_S3].dp = 
+   alert_dialog[A_S1].dp = alert_dialog[A_S2].dp = alert_dialog[A_S3].dp =
    alert_dialog[A_B1].dp = alert_dialog[A_B2].dp = empty_string;
 
    if (s1) {
@@ -2187,12 +2356,12 @@ int alert3(AL_CONST char *s1, AL_CONST char *s2, AL_CONST char *s3, AL_CONST cha
    maxlen += avg_w*4;
    alert_dialog[0].w = maxlen;
    alert_dialog[A_S1].w = alert_dialog[A_S2].w = alert_dialog[A_S3].w = maxlen - avg_w*2;
-   alert_dialog[A_S1].x = alert_dialog[A_S2].x = alert_dialog[A_S3].x = 
+   alert_dialog[A_S1].x = alert_dialog[A_S2].x = alert_dialog[A_S3].x =
 						alert_dialog[0].x + avg_w;
 
    alert_dialog[A_B1].w = alert_dialog[A_B2].w = alert_dialog[A_B3].w = len1;
 
-   alert_dialog[A_B1].x = alert_dialog[A_B2].x = alert_dialog[A_B3].x = 
+   alert_dialog[A_B1].x = alert_dialog[A_B2].x = alert_dialog[A_B3].x =
 				       alert_dialog[0].x + maxlen/2 - len1/2;
 
    if (buttons == 3) {
@@ -2252,4 +2421,3 @@ int alert(AL_CONST char *s1, AL_CONST char *s2, AL_CONST char *s3, AL_CONST char
 
    return ret;
 }
-
