@@ -28,9 +28,10 @@
 
 #include "allegro.h"
 #include "allegro/platform/aintunix.h"
+#include "allegro/internal/aintern.h"
 
 
-#ifdef HAVE_LINUX_JOYSTICK_H
+#ifdef ALLEGRO_HAVE_LINUX_JOYSTICK_H
 
 #include <linux/joystick.h>
 
@@ -136,7 +137,7 @@ static int joy_init(void)
 	    j->stick[s].num_axis = 2;
 	    j->stick[s].axis[0].name = get_config_text("X");
 	    j->stick[s].axis[1].name = get_config_text("Y");
-	    j->stick[s].name = malloc (32);
+	    j->stick[s].name = _AL_MALLOC_ATOMIC(32);
 	    ASSERT(j->stick[s].name);
 	    uszprintf((char *)j->stick[s].name, 32, get_config_text("Stick %d"), s+1);
 	    axis[i][a++] = &j->stick[s].axis[0];
@@ -147,7 +148,7 @@ static int joy_init(void)
       j->num_sticks = s;
 
       for (b = 0; b < num_buttons; b++) {
-	 j->button[b].name = malloc (16);
+	 j->button[b].name = _AL_MALLOC_ATOMIC(16);
 	 ASSERT(j->button[b].name);
 	 uszprintf((char *)j->button[b].name, 16, uconvert_ascii("%c", tmp), 'A' + b);
       }
@@ -173,9 +174,9 @@ static void joy_exit(void)
    for (i = 0; i < num_joysticks; i++) {
       close(joy_fd[i]);
       for (j = 0; j < joy[i].num_sticks; j++)
-	 free((void *)joy[i].stick[j].name);
+	 _AL_FREE((void *)joy[i].stick[j].name);
       for (j = 0; j < joy[i].num_buttons; j++)
-	 free((void *)joy[i].button[j].name);
+	 _AL_FREE((void *)joy[i].button[j].name);
    }
 }
 
@@ -274,7 +275,7 @@ static JOYSTICK_DRIVER joystick_linux_analogue = {
 
 /* list the available drivers */
 _DRIVER_INFO _linux_joystick_driver_list[] = {
-#ifdef HAVE_LINUX_JOYSTICK_H
+#ifdef ALLEGRO_HAVE_LINUX_JOYSTICK_H
 	{    JOY_TYPE_LINUX_ANALOGUE,  &joystick_linux_analogue,  TRUE   },
 #endif
 	{    JOY_TYPE_NONE,            &joystick_none,            TRUE   },
