@@ -29,7 +29,7 @@
 #include "allegro/platform/aintlnx.h"
 
 #ifdef ALLEGRO_LINUX_VGA
-#ifdef HAVE_SYS_IO_H
+#ifdef ALLEGRO_HAVE_SYS_IO_H
 /* iopl() exists in here instead of unistd.h in glibc */
 #include <sys/io.h>
 #endif
@@ -45,6 +45,8 @@
 static int  sys_linux_init(void);
 static void sys_linux_exit(void);
 static void sys_linux_message (AL_CONST char *msg);
+static void sys_linux_save_console_state(void);
+static void sys_linux_restore_console_state(void);
 
 
 /* driver list getters */
@@ -74,8 +76,8 @@ SYSTEM_DRIVER system_linux =
    NULL, /* set_close_button_callback */
    sys_linux_message,
    NULL, /* assert */
-   NULL, /* save_console_state */
-   NULL, /* restore_console_state */
+   sys_linux_save_console_state,
+   sys_linux_restore_console_state,
    NULL, /* create_bitmap */
    NULL, /* created_bitmap */
    NULL, /* create_sub_bitmap */
@@ -86,11 +88,16 @@ SYSTEM_DRIVER system_linux =
    NULL, /* get_vtable */
    __al_linux_set_display_switch_mode,
    __al_linux_display_switch_lock,
+#if (defined ALLEGRO_LINUX_FBCON) && (!defined ALLEGRO_WITH_MODULES)
+   __al_linux_get_fb_color_depth,
+   __al_linux_get_fb_resolution,
+#else
    NULL, /* desktop_color_depth */
    NULL, /* get_desktop_resolution */
+#endif
    NULL, /* get_gfx_safe_mode */
    _unix_yield_timeslice,
-#ifdef HAVE_LIBPTHREAD
+#ifdef ALLEGRO_HAVE_LIBPTHREAD
    _unix_create_mutex,
    _unix_destroy_mutex,
    _unix_lock_mutex,
@@ -134,7 +141,7 @@ static RETSIGTYPE signal_handler (int num)
  */
 static int __al_linux_bgman_init (void)
 {
-#ifdef HAVE_LIBPTHREAD
+#ifdef ALLEGRO_HAVE_LIBPTHREAD
 	_unix_bg_man = &_bg_man_pthreads;
 #else
 	_unix_bg_man = &_bg_man_sigalrm;
@@ -164,6 +171,8 @@ static int sys_linux_init (void)
 	_unix_read_os_type();
 	if (os_type != OSTYPE_LINUX) return -1; /* FWIW */
 
+   _unix_guess_file_encoding();
+
 	/* This is the only bit that needs root privileges.  First
 	 * we attempt to set our euid to 0, in case this is the
 	 * second time we've been called. */
@@ -184,9 +193,6 @@ static int sys_linux_init (void)
 	/* Load dynamic modules */
 	_unix_load_modules(SYSTEM_LINUX);
     
-	/* Initialise the console subsystem */
-	if (__al_linux_init_console()) return -1;
-
 	/* Initialise VGA helpers */
 #ifdef ALLEGRO_LINUX_VGA
 	if (__al_linux_have_ioperms)
@@ -211,10 +217,6 @@ static int sys_linux_init (void)
 		return -1;
 	}
 
-	/* Initialise the console switching system */
-	set_display_switch_mode (SWITCH_PAUSE);
-	__al_linux_init_vtswitch();
-
 	return 0;
 }
 
@@ -225,9 +227,6 @@ static int sys_linux_init (void)
  */
 static void sys_linux_exit (void)
 {
-	/* shut down the console switching system */
-	__al_linux_done_vtswitch();
-
 	/* shut down asynchronous event processing */
 	__al_linux_bgman_exit();
 
@@ -248,9 +247,6 @@ static void sys_linux_exit (void)
 		__al_linux_shutdown_vga_helpers();
 #endif
 
-	/* shut down the console subsystem */
-	__al_linux_done_console();
-
 	/* unload dynamic modules */
 	_unix_unload_modules();
 
@@ -265,6 +261,20 @@ static void sys_linux_exit (void)
 
 
 
+static void sys_linux_save_console_state(void)
+{
+   __al_linux_use_console();
+}
+
+
+
+static void sys_linux_restore_console_state(void)
+{
+   __al_linux_leave_console();
+}
+
+
+
 /* sys_linux_message:
  *  Display a message on our original console.
  */
@@ -274,7 +284,7 @@ static void sys_linux_message (AL_CONST char *msg)
    int ret;
    ASSERT(msg);
 
-   tmp = malloc(ALLEGRO_MESSAGE_SIZE);
+   tmp = _AL_MALLOC_ATOMIC(ALLEGRO_MESSAGE_SIZE);
    msg = uconvert(msg, U_CURRENT, tmp, U_ASCII, ALLEGRO_MESSAGE_SIZE);
 
    do {
@@ -285,7 +295,7 @@ static void sys_linux_message (AL_CONST char *msg)
 
    __al_linux_got_text_message = TRUE;
 
-   free(tmp);
+   _AL_FREE(tmp);
 }
 
 
