@@ -33,7 +33,11 @@ static void hid_scan_application_collection(CFMutableDictionaryRef properties, H
 
 static HID_DEVICE* add_device(HID_DEVICE_COLLECTION*);
 
-
+/* add_element:
+ * Add a new, blank element to a device.
+ * This is later specialized into a button
+ * axis or whatever 
+ */
 static HID_ELEMENT* add_element(HID_DEVICE* d) {
    HID_ELEMENT* e;
    if (d->element==NULL) {
@@ -48,15 +52,26 @@ static HID_ELEMENT* add_element(HID_DEVICE* d) {
    memset(e, 0, sizeof(HID_ELEMENT));
    return e;
 }
+
 /* i_val:
  * Helper method - get an integer value from a dictionary
+ * Defaults to 0 if the value is not found; in practice this 
+ * should not occur.
  */
 static int i_val(CFTypeRef d, CFStringRef key) {
    int ans;
-   CFNumberGetValue(CFDictionaryGetValue(d, key), kCFNumberIntType, &ans);
+   CFTypeRef num = CFDictionaryGetValue(d, key);
+   if (num) 
+      CFNumberGetValue(num, kCFNumberIntType, &ans);
+   else
+      ans = 0;
    return ans;
 }
 
+/* hid_store_element_data:
+ * Parse this HID element, break it down and store the 
+ * relevant data in the device structure
+ */
 static void hid_store_element_data(CFTypeRef element, int type, HID_DEVICE *device, int app, int col, int idx)
 {
    HID_ELEMENT *hid_element;
@@ -65,12 +80,12 @@ static void hid_store_element_data(CFTypeRef element, int type, HID_DEVICE *devi
 
    hid_element = add_element(device);
    hid_element->type = type;
-   hid_element->index=idx;
-   hid_element->col=col;
-   hid_element->app=app;
-   hid_element->cookie=(IOHIDElementCookie) i_val(element, CFSTR(kIOHIDElementCookieKey));
-   hid_element->min=i_val(element, CFSTR(kIOHIDElementMinKey));
-   hid_element->max=i_val(element, CFSTR(kIOHIDElementMaxKey));
+   hid_element->index = idx;
+   hid_element->col = col;
+   hid_element->app = app;
+   hid_element->cookie = (IOHIDElementCookie) i_val(element, CFSTR(kIOHIDElementCookieKey));
+   hid_element->min = i_val(element, CFSTR(kIOHIDElementMinKey));
+   hid_element->max = i_val(element, CFSTR(kIOHIDElementMaxKey));
    type_ref = CFDictionaryGetValue(element, CFSTR(kIOHIDElementNameKey));
    if ((type_ref) && (name = CFStringGetCStringPtr(type_ref, CFStringGetSystemEncoding())))
       hid_element->name = strdup(name);
@@ -78,10 +93,9 @@ static void hid_store_element_data(CFTypeRef element, int type, HID_DEVICE *devi
       hid_element->name = NULL;
 }
 
-
-
 /* hid_scan_application:
  * scan the elements that make up one 'application'
+ * i.e. one unit like a joystick.
  */
 static void hid_scan_application(CFTypeRef properties, HID_DEVICE *device, int app)
 {
@@ -95,55 +109,75 @@ static void hid_scan_application(CFTypeRef properties, HID_DEVICE *device, int a
       for (i = 0; i < CFArrayGetCount(array_ref); i++) {
          element = CFArrayGetValueAtIndex(array_ref, i);
          if (CFGetTypeID(element) == CFDictionaryGetTypeID()) {
-            type=i_val(element, CFSTR(kIOHIDElementTypeKey));
-            usage=i_val(element, CFSTR(kIOHIDElementUsageKey));
-            usage_page=i_val(element, CFSTR(kIOHIDElementUsagePageKey));
-            switch (usage_page) {
-            case kHIDPage_GenericDesktop:
-               switch(usage) {
-               case kHIDUsage_GD_Pointer:
-                  if (axis!=0) {
-                     /* already have some elements in this stick */
-                     ++stick;
-                     axis=0;
+            type = i_val(element, CFSTR(kIOHIDElementTypeKey));
+            usage = i_val(element, CFSTR(kIOHIDElementUsageKey));
+            usage_page = i_val(element, CFSTR(kIOHIDElementUsagePageKey));
+            if (type == kIOHIDElementTypeCollection) 
+               {
+                  /* It is a collection; recurse into it, if it is
+                   part of the generic desktop (sometimes the whole
+                   joystick is wrapped up inside a collection like
+                   this. */
+                  if (usage_page == kHIDPage_GenericDesktop)
+                     hid_scan_application(element, device, app);                     
+               }
+            else
+               { 
+                  switch (usage_page) {
+                  case kHIDPage_GenericDesktop:
+                     switch(usage) {
+                     case kHIDUsage_GD_Pointer:
+                        if (axis!=0) {
+                           /* already have some elements in this stick */
+                           ++stick;
+                           axis=0;
+                        }
+                        hid_scan_physical_collection(element, device, app, stick);
+                        ++stick;
+                        break;
+                     case kHIDUsage_GD_X:
+                        hid_store_element_data(element, HID_ELEMENT_AXIS_PRIMARY_X, device, app, stick, axis++);
+                        break;
+                        
+                     case kHIDUsage_GD_Y:
+                        hid_store_element_data(element, HID_ELEMENT_AXIS_PRIMARY_Y, device, app, stick, axis++);
+                        break;
+                        
+                     case kHIDUsage_GD_Z:
+                     case kHIDUsage_GD_Rx:
+                     case kHIDUsage_GD_Ry:
+                     case kHIDUsage_GD_Rz:
+                        hid_store_element_data(element, HID_ELEMENT_AXIS, device,app, stick, axis++);
+                        break;
+                        
+                     case kHIDUsage_GD_Slider:
+                     case kHIDUsage_GD_Dial:
+                     case kHIDUsage_GD_Wheel:
+                        /* If we've already seen some axes on this stick, move to the next one */
+                        if (axis > 0) 
+                           {
+                              ++stick;
+                              axis=0;
+                           }
+                        hid_store_element_data(element, HID_ELEMENT_STANDALONE_AXIS, device, app, stick++, 0);
+                        break;
+                    
+                     case kHIDUsage_GD_Hatswitch:
+                        /* If we've already seen some axes on this stick, move to the next one */
+                        if (axis > 0) 
+                           {
+                              ++stick;
+                              axis=0;
+                           }
+                        hid_store_element_data(element, HID_ELEMENT_HAT, device, app, stick++, 0);
+                        break;
+                     }
+                     break;
+                  case kHIDPage_Button:
+                     hid_store_element_data(element, HID_ELEMENT_BUTTON, device, app, 0, usage-1);
+                     break;
                   }
-                  hid_scan_physical_collection(element, device, app, stick);
-                  ++stick;
-                  break;
-               case kHIDUsage_GD_X:
-                  hid_store_element_data(element, HID_ELEMENT_AXIS_PRIMARY_X, device, app, stick, axis++);
-                  break;
-                    
-               case kHIDUsage_GD_Y:
-                  hid_store_element_data(element, HID_ELEMENT_AXIS_PRIMARY_Y, device, app, stick, axis++);
-                  break;
-                    
-               case kHIDUsage_GD_Z:
-               case kHIDUsage_GD_Rx:
-               case kHIDUsage_GD_Ry:
-               case kHIDUsage_GD_Rz:
-                  hid_store_element_data(element, HID_ELEMENT_AXIS, device,app, stick, axis++);
-                  break;
-                    
-               case kHIDUsage_GD_Slider:
-               case kHIDUsage_GD_Dial:
-               case kHIDUsage_GD_Wheel:
-                  ++stick;
-                  axis=0;
-                  hid_store_element_data(element, HID_ELEMENT_STANDALONE_AXIS, device, app, stick++, 0);
-                  break;
-                    
-               case kHIDUsage_GD_Hatswitch:
-                  ++stick;
-                  axis=0;
-                  hid_store_element_data(element, HID_ELEMENT_HAT, device, app, stick++, 0);
-                  break;
                }
-               break;
-            case kHIDPage_Button:
-               hid_store_element_data(element, HID_ELEMENT_BUTTON, device, app, 0, usage-1);
-               break;
-            }
             if (axis>=2) {
                ++stick;
                axis=0;
@@ -167,9 +201,9 @@ static void hid_scan_physical_collection(CFTypeRef properties, HID_DEVICE *devic
       for (i = 0; i < CFArrayGetCount(array_ref); i++) {
          element = CFArrayGetValueAtIndex(array_ref, i);
          if (CFGetTypeID(element) == CFDictionaryGetTypeID()) {
-            type=i_val(element, CFSTR(kIOHIDElementTypeKey));
-            usage=i_val(element, CFSTR(kIOHIDElementUsageKey));
-            usage_page=i_val(element, CFSTR(kIOHIDElementUsagePageKey));
+            type = i_val(element, CFSTR(kIOHIDElementTypeKey));
+            usage = i_val(element, CFSTR(kIOHIDElementUsageKey));
+            usage_page = i_val(element, CFSTR(kIOHIDElementUsagePageKey));
             switch (usage_page) {
             case kHIDPage_GenericDesktop:
                switch(usage) {
@@ -239,6 +273,62 @@ static void hid_scan_application_collection(CFMutableDictionaryRef properties, H
    }
 }
 
+/* get_usb_properties:
+ * Get a property dictionary from the USB plane. 
+ */
+static CFMutableDictionaryRef get_usb_properties(io_object_t device)
+{
+   io_registry_entry_t parent, grandparent;
+   CFMutableDictionaryRef usb_properties = NULL;
+   if (IORegistryEntryGetParentEntry(device, kIOServicePlane, &parent) == KERN_SUCCESS) {
+      if (IORegistryEntryGetParentEntry(parent, kIOServicePlane, &grandparent) == KERN_SUCCESS) {
+         IORegistryEntryCreateCFProperties (grandparent, &usb_properties, 
+                                            kCFAllocatorDefault, kNilOptions);
+         IOObjectRelease(grandparent);
+      }
+      IOObjectRelease(parent);
+   }
+   return usb_properties;
+}
+
+#if OSX_HID_PSEUDO_SCAN
+/*
+ * Pseudo scan - for development purposes, if someone has hardware
+ * that isn't parsed by this code, you can ask them to dump _their_ scan
+ * as a plist, then reload it here in order to debug it.
+*/
+HID_DEVICE_COLLECTION *osx_hid_scan(int type, HID_DEVICE_COLLECTION* col)
+{
+   HID_DEVICE* this_device;
+   NSDictionary* properties = nil;
+   NSString* filename;
+   switch (type) {
+   case HID_GAMEPAD:
+      filename = @"gamepad.plist";
+      break;
+   case HID_JOYSTICK:
+      filename = @"joystick.plist";
+      break;
+   case HID_MOUSE:
+      filename = @"mouse.plist";
+      break;
+   default:
+      filename = nil;
+      break;
+   }
+   if (filename != nil)
+      properties = [NSDictionary dictionaryWithContentsOfFile:filename];
+   if (properties)
+   {   
+      this_device = add_device(col);
+      this_device->manufacturer = strdup([((NSString*) [properties objectForKey: @kIOHIDManufacturerKey]) lossyCString]);
+      this_device->product = strdup([((NSString*) [properties objectForKey: @kIOHIDProductKey]) lossyCString]);
+      hid_scan_application_collection((CFDictionaryRef) properties, this_device);
+      [properties release];
+   }
+   return col;
+}
+#else
 /* osx_hid_scan:
  * Scan the hid manager for devices of type 'type',
  * and append to the collection col
@@ -250,7 +340,6 @@ HID_DEVICE_COLLECTION *osx_hid_scan(int type, HID_DEVICE_COLLECTION* col)
    mach_port_t master_port = 0;
    io_iterator_t hid_object_iterator = 0;
    io_object_t hid_device = 0;
-   io_registry_entry_t parent1, parent2;
    CFMutableDictionaryRef class_dictionary = NULL;
    int usage, usage_page;
    int app_count;
@@ -289,28 +378,22 @@ HID_DEVICE_COLLECTION *osx_hid_scan(int type, HID_DEVICE_COLLECTION* col)
       result = IOServiceGetMatchingServices(master_port, class_dictionary, &hid_object_iterator);
       if ((result == kIOReturnSuccess) && (hid_object_iterator)) {
          /* Ok, we have a list of attached HID devices; scan them. */
-         while ((hid_device = IOIteratorNext(hid_object_iterator))!=NULL) {
-            /* Mac OS X currently is not mirroring all USB properties to HID page so need to look at USB device page also
-             * get dictionary for usb properties: step up two levels and get CF dictionary for USB properties.
-             */
-            if ((IORegistryEntryCreateCFProperties(hid_device, &properties, kCFAllocatorDefault, kNilOptions) == KERN_SUCCESS) &&
-                (IORegistryEntryGetParentEntry(hid_device, kIOServicePlane, &parent1) == KERN_SUCCESS) &&
-                (IORegistryEntryGetParentEntry (parent1, kIOServicePlane, &parent2) == KERN_SUCCESS) &&
-                (IORegistryEntryCreateCFProperties (parent2, &usb_properties, kCFAllocatorDefault, kNilOptions) == KERN_SUCCESS) &&
-                (properties) && (usb_properties)) {
-               IOObjectRelease(parent1);
-               IOObjectRelease(parent2);
+         while ((hid_device = IOIteratorNext(hid_object_iterator))!=0) {
+            if ((IORegistryEntryCreateCFProperties(hid_device, &properties, kCFAllocatorDefault, kNilOptions) == KERN_SUCCESS) && (properties != NULL)) {
                error = FALSE;
                this_device = add_device(col);
                this_device->type = type;
                if (col->count==0) {
                   this_device->cur_app=0;
                }
-               else
-                  {
-                     this_device->cur_app=col->devices[col->count-1].cur_app;
-                  }
-
+               else {
+                  this_device->cur_app=col->devices[col->count-1].cur_app;
+               }
+               /* 
+                * Mac OS X currently is not mirroring all USB properties
+                * to HID page so need to look at USB device page also. 
+                */
+               usb_properties = get_usb_properties(hid_device);
                type_ref = CFDictionaryGetValue(properties, CFSTR(kIOHIDManufacturerKey));
                if (!type_ref)
                   type_ref = CFDictionaryGetValue(usb_properties, CFSTR("USB Vendor Name"));
@@ -374,7 +457,7 @@ HID_DEVICE_COLLECTION *osx_hid_scan(int type, HID_DEVICE_COLLECTION* col)
 
    return col;
 }
-
+#endif
 /* add_device:
  * add a new device to a collection
  */
@@ -389,8 +472,12 @@ static HID_DEVICE* add_device(HID_DEVICE_COLLECTION* o) {
       o->capacity*=2;
       o->devices=realloc(o->devices, o->capacity*sizeof(HID_DEVICE));
    }
-   d=&o->devices[o->count++];
+   d=&o->devices[o->count];
    memset(d, 0, sizeof(HID_DEVICE));
+   /* Chain onto the preceding app count */
+   if (o->count>0)
+      d->cur_app = o->devices[o->count-1].cur_app;
+   ++o->count;
    return d;
 }
 
