@@ -115,7 +115,7 @@ void calc_spline(AL_CONST int points[8], int npts, int *out_x, int *out_y)
 /* spline:
  *  Draws a bezier spline onto the specified bitmap in the specified color.
  */
-void spline(BITMAP *bmp, AL_CONST int points[8], int color)
+void _soft_spline(BITMAP *bmp, AL_CONST int points[8], int color)
 {   
    #define MAX_POINTS   64
 
