@@ -95,7 +95,8 @@ static char alsa_desc[256] = EMPTY_STRING;
 static int alsa_detect(int input);
 static int alsa_init(int input, int voices);
 static void alsa_exit(int input);
-static int alsa_mixer_volume(int volume);
+static int alsa_set_mixer_volume(int volume);
+static int alsa_get_mixer_volume(void);
 static int alsa_buffer_size(void);
 
 
@@ -114,7 +115,8 @@ DIGI_DRIVER digi_alsa =
    alsa_detect,
    alsa_init,
    alsa_exit,
-   alsa_mixer_volume,
+   alsa_set_mixer_volume,
+   alsa_get_mixer_volume,
 
    NULL,
    NULL,
@@ -361,7 +363,7 @@ static int alsa_init(int input, int voices)
 	 alsa_bits = 8;
 	 break;
 
-      case SND_PCM_FORMAT_U16_LE:
+      case SND_PCM_FORMAT_U16_NE:
 	 if (sizeof(short) != 2) {
 	    ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Unsupported sample format"));
 	    goto Error;
@@ -408,7 +410,7 @@ static int alsa_init(int input, int voices)
    ALSA9_CHECK(snd_pcm_sw_params(pcm_handle, swparams));
 
    /* Allocate mixing buffer. */
-   alsa_bufdata = malloc(alsa_bufsize * alsa_sample_size);
+   alsa_bufdata = _AL_MALLOC_ATOMIC(alsa_bufsize * alsa_sample_size);
    if (!alsa_bufdata) {
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Can not allocate audio buffer"));
       goto Error;
@@ -431,7 +433,7 @@ static int alsa_init(int input, int voices)
       goto Error;
    }
 
-   ufds = malloc(sizeof(struct pollfd) * pdc);
+   ufds = _AL_MALLOC(sizeof(struct pollfd) * pdc);
    if (ufds == NULL) {
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Not enough memory for poll descriptors"));
       goto Error;
@@ -476,7 +478,7 @@ static void alsa_exit(int input)
 
    _unix_bg_man->unregister_func(alsa_update);
 
-   free(alsa_bufdata);
+   _AL_FREE(alsa_bufdata);
    alsa_bufdata = NULL;
 
    _mixer_exit();
@@ -492,10 +494,10 @@ static void alsa_exit(int input)
 
 
 
-/* alsa_mixer_volume:
+/* alsa_set_mixer_volume:
  *  Set mixer volume (0-255)
  */
-static int alsa_mixer_volume(int volume)
+static int alsa_set_mixer_volume(int volume)
 {
    if (alsa_mixer && alsa_mixer_elem) {
       snd_mixer_selem_set_playback_volume(alsa_mixer_elem, 0, volume * alsa_mixer_allegro_ratio);
@@ -506,6 +508,32 @@ static int alsa_mixer_volume(int volume)
 
 
 
+/* alsa_get_mixer_volume:
+ *  Return mixer volume (0-255)
+ */
+static int alsa_get_mixer_volume(void)
+{
+   if (alsa_mixer && alsa_mixer_elem) {
+      long vol1, vol2;
+
+      snd_mixer_handle_events(alsa_mixer);
+
+      if (snd_mixer_selem_get_playback_volume(alsa_mixer_elem, 0, &vol1) < 0)
+	 return -1;
+      if (snd_mixer_selem_get_playback_volume(alsa_mixer_elem, 1, &vol2) < 0)
+         return -1;
+
+      vol1 /= alsa_mixer_allegro_ratio;
+      vol2 /= alsa_mixer_allegro_ratio;
+
+      return (vol1 + vol2) / 2;
+   }
+
+   return -1;
+}
+
+
+
 #ifdef ALLEGRO_MODULE
 
 /* _module_init:
