@@ -60,7 +60,7 @@ static int jack_detect(int input);
 static int jack_init(int input, int voices);
 static void jack_exit(int input);
 static int jack_buffer_size(void);
-static int jack_mixer_volume(int volume);
+static int jack_set_mixer_volume(int volume);
 
 DIGI_DRIVER digi_jack =
 {
@@ -76,7 +76,8 @@ DIGI_DRIVER digi_jack =
    jack_detect,
    jack_init,
    jack_exit,
-   jack_mixer_volume,
+   jack_set_mixer_volume,
+   NULL,
 
    NULL,
    NULL,
@@ -239,7 +240,7 @@ static int jack_init(int input, int voices)
 
    jack_rate = jack_get_sample_rate (jack_client);
 
-   jack_buffer = malloc(jack_bufsize * (1 + jack_16bit) * (1 + jack_stereo));
+   jack_buffer = _AL_MALLOC_ATOMIC(jack_bufsize * (1 + jack_16bit) * (1 + jack_stereo));
    if (!jack_buffer) {
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text(
          "Cannot allocate audio buffer"));
@@ -283,7 +284,7 @@ static int jack_init(int input, int voices)
 	 if (jack_connect (jack_client, jack_port_name (output_right), ports[1]) == 0)
 	    TRACE (PREFIX_I "Connected right playback port to %s", ports[1]);
       }
-      free (ports);
+      _AL_FREE (ports);
    }
 
    uszprintf(jack_desc, sizeof(jack_desc),
@@ -308,10 +309,10 @@ static void jack_exit(int input)
 
 
 
-/* jack_mixer_volume:
+/* jack_set_mixer_volume:
  *  Set mixer volume (0-255)
  */
-static int jack_mixer_volume(int volume)
+static int jack_set_mixer_volume(int volume)
 {
    /* Not implemented */
    return 0;
