@@ -35,13 +35,13 @@
 #include <sys/stat.h>
 #include <fcntl.h>
 #include <string.h>
-#if defined(HAVE_SOUNDCARD_H)
+#if defined(ALLEGRO_HAVE_SOUNDCARD_H)
    #include <soundcard.h>
-#elif defined(HAVE_SYS_SOUNDCARD_H)
+#elif defined(ALLEGRO_HAVE_SYS_SOUNDCARD_H)
    #include <sys/soundcard.h>
-#elif defined(HAVE_MACHINE_SOUNDCARD_H)
+#elif defined(ALLEGRO_HAVE_MACHINE_SOUNDCARD_H)
    #include <machine/soundcard.h>
-#elif defined(HAVE_LINUX_SOUNDCARD_H)
+#elif defined(ALLEGRO_HAVE_LINUX_SOUNDCARD_H)
    #include <linux/soundcard.h>
 #endif
 #include <sys/ioctl.h>
@@ -83,7 +83,8 @@ static int oss_rec_bufsize;
 static int oss_detect(int input);
 static int oss_init(int input, int voices);
 static void oss_exit(int input);
-static int oss_mixer_volume(int volume);
+static int oss_set_mixer_volume(int volume);
+static int oss_get_mixer_volume(void);
 static int oss_buffer_size(void);
 static int oss_rec_cap_rate(int bits, int stereo);
 static int oss_rec_cap_parm(int rate, int bits, int stereo);
@@ -108,7 +109,8 @@ DIGI_DRIVER digi_oss =
    oss_detect,
    oss_init,
    oss_exit,
-   oss_mixer_volume,
+   oss_set_mixer_volume,
+   oss_get_mixer_volume,
 
    NULL,
    NULL,
@@ -372,7 +374,7 @@ static int oss_init(int input, int voices)
    }
 
    oss_bufsize = bufinfo.fragsize;
-   oss_bufdata = malloc(oss_bufsize);
+   oss_bufdata = _AL_MALLOC_ATOMIC(oss_bufsize);
 
    if (oss_bufdata == 0) {
       ustrzcpy(allegro_error, ALLEGRO_ERROR_SIZE, get_config_text("Can not allocate audio buffer"));
@@ -419,7 +421,7 @@ static void oss_exit(int input)
 
    _unix_bg_man->unregister_func(oss_update);
 
-   free(oss_bufdata);
+   _AL_FREE(oss_bufdata);
    oss_bufdata = 0;
 
    _mixer_exit();
@@ -429,10 +431,10 @@ static void oss_exit(int input)
 
 
 
-/* oss_mixer_volume:
+/* oss_set_mixer_volume:
  *  Set mixer volume.
  */
-static int oss_mixer_volume(int volume)
+static int oss_set_mixer_volume(int volume)
 {
    int fd, vol, ret;
    char tmp[128];
@@ -451,6 +453,29 @@ static int oss_mixer_volume(int volume)
 
 
 
+/* oss_get_mixer_volume:
+ *  Return mixer volume.
+ */
+static int oss_get_mixer_volume(void)
+{
+   int fd, vol;
+   char tmp[128];
+
+   fd = open(uconvert_toascii(_oss_mixer_driver, tmp), O_RDONLY);
+   if (fd < 0)
+      return -1;
+
+   if (ioctl(fd, MIXER_READ(SOUND_MIXER_PCM), &vol) != 0)
+      return -1;
+   close(fd);
+
+   vol = ((vol & 0xff) + (vol >> 8)) / 2;
+   vol = vol * 255 / 100;
+   return vol;
+}
+
+
+
 /* oss_rec_cap_rate:
  *  Returns maximum input sampling rate.
  */
