@@ -53,7 +53,8 @@
 static int digi_dsoundmix_detect(int input);
 static int digi_dsoundmix_init(int input, int voices);
 static void digi_dsoundmix_exit(int input);
-static int digi_dsoundmix_mixer_volume(int volume);
+static int digi_dsoundmix_set_mixer_volume(int volume);
+static int digi_dsoundmix_get_mixer_volume(void);
 static int digi_dsoundmix_buffer_size(void);
 
 
@@ -73,7 +74,8 @@ static DIGI_DRIVER digi_dsoundmix =
    digi_dsoundmix_detect,
    digi_dsoundmix_init,
    digi_dsoundmix_exit,
-   digi_dsoundmix_mixer_volume,
+   digi_dsoundmix_set_mixer_volume,
+   digi_dsoundmix_get_mixer_volume,
 
    /* audiostream locking functions */
    NULL,  // AL_METHOD(void *, lock_voice, (int voice, int start, int end));
@@ -216,7 +218,7 @@ DIGI_DRIVER *_get_dsalmix_driver(char *name, LPGUID guid, int num)
 {
    DIGI_DRIVER *driver;
 
-   driver = malloc(sizeof(DIGI_DRIVER));
+   driver = _AL_MALLOC(sizeof(DIGI_DRIVER));
    if (!driver)
       return NULL;
 
@@ -224,7 +226,7 @@ DIGI_DRIVER *_get_dsalmix_driver(char *name, LPGUID guid, int num)
 
    driver->id = DIGI_DIRECTAMX(num);
 
-   driver_names[num] = malloc(strlen(name)+10);
+   driver_names[num] = _AL_MALLOC_ATOMIC(strlen(name)+10);
    if (driver_names[num]) {
       _al_sane_strncpy(driver_names[num], "Allegmix ", strlen(name)+10);
       _al_sane_strncpy(driver_names[num]+9, name, strlen(name)+1);
@@ -609,7 +611,7 @@ static int digi_dsoundmix_init(int input, int voices)
 #if USE_NEW_CODE
    digidsbufsize /= 2;
 #endif
-   digidsbufdata = malloc(digidsbufsize);
+   digidsbufdata = _AL_MALLOC_ATOMIC(digidsbufsize);
    if (!digidsbufdata) {
       _TRACE(PREFIX_E "Can't create temp buffer\n");
       return -1;
@@ -666,7 +668,7 @@ static void digi_dsoundmix_exit(int input)
    remove_int(digi_dsoundmix_mixer_callback);
 
    if (digidsbufdata) {
-      free(digidsbufdata);
+      _AL_FREE(digidsbufdata);
       digidsbufdata = NULL;
    }
 
@@ -694,13 +696,13 @@ static void digi_dsoundmix_exit(int input)
 
 
 
-/* digi_dsoundmix_mixer_volume:
+/* digi_dsoundmix_set_mixer_volume:
  */
-static int digi_dsoundmix_mixer_volume(int volume)
+static int digi_dsoundmix_set_mixer_volume(int volume)
 {
    if (prim_buf) {
       alleg_buf_vol = alleg_to_dsound_volume[MID(0, volume, 255)];
-      IDirectSoundBuffer_SetVolume(alleg_buf, alleg_buf_vol); 
+      IDirectSoundBuffer_SetVolume(alleg_buf, alleg_buf_vol);
    }
 
    return 0;
@@ -708,6 +710,23 @@ static int digi_dsoundmix_mixer_volume(int volume)
 
 
 
+/* digi_dsoundmix_get_mixer_volume:
+ */
+static int digi_dsoundmix_get_mixer_volume(void)
+{
+   LONG vol;
+
+   if (!prim_buf)
+     return -1;
+
+   IDirectSoundBuffer_GetVolume(alleg_buf, &vol);
+   vol = MID(0, pow(10, (vol/2000.0))*255.0 - DSBVOLUME_MAX, 255);
+
+   return vol;
+}
+
+
+
 /* digi_dsoundmix_buffer_size:
  */
 static int digi_dsoundmix_buffer_size(void)
