@@ -47,7 +47,8 @@
 static int digi_waveout_detect(int input);
 static int digi_waveout_init(int input, int voices);
 static void digi_waveout_exit(int input);
-static int digi_waveout_mixer_volume(int volume);
+static int digi_waveout_set_mixer_volume(int volume);
+static int digi_waveout_get_mixer_volume(void);
 static int digi_waveout_buffer_size(void);
 
 
@@ -67,7 +68,8 @@ static DIGI_DRIVER digi_waveout =
    digi_waveout_detect,
    digi_waveout_init,
    digi_waveout_exit,
-   digi_waveout_mixer_volume,
+   digi_waveout_set_mixer_volume,
+   digi_waveout_get_mixer_volume,
 
    /* audiostream locking functions */
    NULL,  // AL_METHOD(void *, lock_voice, (int voice, int start, int end));
@@ -140,7 +142,7 @@ DIGI_DRIVER *_get_woalmix_driver(int num)
 {
    DIGI_DRIVER *driver;
 
-   driver = malloc(sizeof(DIGI_DRIVER));
+   driver = _AL_MALLOC(sizeof(DIGI_DRIVER));
    if (!driver)
       return NULL;
 
@@ -264,9 +266,9 @@ static int digi_waveout_init(int input, int voices)
       digiwobufsize = digiwobufdivs * 512;
    }
 
-   digiwobufdata = malloc(digiwobufsize);
+   digiwobufdata = _AL_MALLOC_ATOMIC(digiwobufsize);
    if (!digiwobufdata) {
-      _TRACE (PREFIX_E "malloc() failed\n");
+      _TRACE (PREFIX_E "_AL_MALLOC_ATOMIC() failed\n");
       goto Error;
    }
 
@@ -283,7 +285,7 @@ static int digi_waveout_init(int input, int voices)
       goto Error;
    }
 
-   lpWaveHdr = malloc(sizeof(WAVEHDR));
+   lpWaveHdr = _AL_MALLOC(sizeof(WAVEHDR));
    lpWaveHdr->lpData = digiwobufdata;
    lpWaveHdr->dwBufferLength = digiwobufsize;
    lpWaveHdr->dwFlags = WHDR_BEGINLOOP | WHDR_ENDLOOP;
@@ -344,21 +346,21 @@ static void digi_waveout_exit(int input)
    }
 
    if (lpWaveHdr) {
-      free(lpWaveHdr);
+      _AL_FREE(lpWaveHdr);
       lpWaveHdr = NULL;
    }
 
    if (digiwobufdata) {
-      free(digiwobufdata);
+      _AL_FREE(digiwobufdata);
       digiwobufdata = NULL;
    }
 }
 
 
 
-/* digi_waveout_mixer_volume:
+/* digi_waveout_set_mixer_volume:
  */
-static int digi_waveout_mixer_volume(int volume)
+static int digi_waveout_set_mixer_volume(int volume)
 {
    DWORD realvol;
 
@@ -374,6 +376,24 @@ static int digi_waveout_mixer_volume(int volume)
 
 
 
+/* digi_waveout_get_mixer_volume:
+ */
+static int digi_waveout_get_mixer_volume(void)
+{
+   DWORD vol;
+   
+   if (!hWaveOut)
+      return -1;
+
+   if (waveOutGetVolume(hWaveOut, &vol) != MMSYSERR_NOERROR)
+      return -1;
+
+   vol &= 0xffff;
+   return vol / (0xffff / 255);
+}
+
+
+
 /* digi_waveout_buffer_size:
  */
 static int digi_waveout_buffer_size(void)
